GPTModel is a Pre-trained Generative Model created by Sapiens Technology®️ and aims at the practical development of large-scale language models. Its code allows for fast, easy and objective coding, with a focus on productivity and high performance.

# GPTModel

The GPTModel (Generative Pre-trained Model with Transformer) is an architecture for developing language models that uses the Transformer neural network architecture as a base. The Transformer is used for pattern recognition through attention mechanisms and later for the generation of intelligent text resulting from auto-regressive processes. All the Transformer code contained in this algorithm was based on the article "Attention Is All You Need", published by Google researchers in 2017. You can check out this full study by clicking [here](https://arxiv.org/abs/1706.03762). No structural modifications were made to the Transformer logic in question, only small optimizations to facilitate training, saving, loading, fine-tuning and inference of the pre-trained model.

Click [here](https://arxiv.org/pdf/1706.03762) to access the full study.

## Installation

Use the package manager [pip](https://pip.pypa.io/en/stable/) to install the GPT Model.

```bash
pip install gptmodel
```

## Usage
Basic usage example:
```python
from gptmodel import GPTModel # importing the main class from the module
gptmodel = GPTModel() # instantiation of the main class object
# the "addFit" method takes an input sample in the first argument and a desired output for this sample in the second argument
# when the "addFit" method is called before the "train" method, it adds the input and output pairs to the dataset for training
# if the dataset is empty, the input and output pairs of the "addFit" method will become the dataset itself
# if the dataset has content, the input and output pairs will be added as an addendum at the end of the dataset
# if the "addFit" method is invoked after training, the sample pairs will be adjusted as a fine-tuning on top of the base model that has already been trained
# the "prompt" argument receives an question example, instruction, or sample of input
# the "answer" argument receives an example answer or sample of output for the input of the same method invocation
# the "addFit" method can be invoked/called as many times as necessary depending on the size of your training dataset
gptmodel.addFit(prompt='What does the acronym GPT mean in language models?', answer='GPT in language models stands for Generative Pre-trained Transformer.') # always use string values in "prompt" and "answer"
gptmodel.addFit(prompt='What is the capital of Spain?', answer='The capital of Spain is Madrid.') # returns True if the addition is successful or False otherwise
gptmodel.train() # the "train" method trains the model with the previously added data
# the "predict" function returns a text with the answer to the input received in the "prompt" argument
# the answer will always be based on the most likely output sample for the input of prediction
# to calculate the response, a probabilistic search will be performed on the pairs added with "addFit" to find the pair that has the training input most similar to the prediction input
# the pair that has the training input closest to the input of the "predict" function will have its response used as the basis for constructing the inference response
answer = gptmodel.predict(prompt='what does gpt mean in language models?') # returns a string with the result of the inference (always use string values in "prompt")
print(answer) # displays the inference result

```
Prediction response:
During training, you can view the backpropagation epoch and see how many repetitions are being performed to adjust the weights. You can also view the current "precision" for each epoch. The "precision" value is calculated based on how much information is being abstracted without loss. You can also view the "precision" you are aiming for and the percentage progress of each backpropagation.
Note: the "precision" referred to is not statistical accuracy in the mathematical sense of the word, it does not refer to a statistical metric, but rather how much information the model is able to learn from the data provided.
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.0it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.1it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0584; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000026 - current precision is 0.1395; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.1it/s]
Backpropagation epoch: 0000000027 - current precision is 0.2492; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000028 - current precision is 0.3104; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.9it/s]
Backpropagation epoch: 0000000029 - current precision is 0.3800; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000030 - current precision is 0.4411; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000031 - current precision is 0.5056; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000032 - current precision is 0.5562; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000033 - current precision is 0.5967; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000034 - current precision is 0.6481; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000035 - current precision is 0.6857; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000036 - current precision is 0.7047; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000037 - current precision is 0.7569; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000038 - current precision is 0.7574; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.1it/s]
Backpropagation epoch: 0000000039 - current precision is 0.7787; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000040 - current precision is 0.8174; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000041 - current precision is 0.7760; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.1it/s]
Backpropagation epoch: 0000000042 - current precision is 0.8289; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000043 - current precision is 0.8106; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.1it/s]
Backpropagation epoch: 0000000044 - current precision is 0.8445; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000045 - current precision is 0.8821; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000046 - current precision is 0.8774; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000047 - current precision is 0.8750; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000048 - current precision is 0.8966; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000049 - current precision is 0.8389; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000050 - current precision is 0.9302; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 47.7kit/s]

GPT in language models stands for Generative Pre-trained Transformer.
```
Here's another basic example with more input and output samples. The more question variations per answer you add, the lower your chances of error in inference.
```python
# note how building language models with the sapiens gpt architecture is much simpler and more objective than if you were using any other module for transformers
from gptmodel import GPTModel
gptmodel = GPTModel()
# we recommend that you use different questions for the same answer, this way you will reduce the chances of error in the inference answers
gptmodel.addFit(prompt='What does the acronym GPT mean in language models?', answer='GPT in language models stands for Generative Pre-trained Transformer.')
gptmodel.addFit(prompt='Tell me what is the meaning of GPT in language models.', answer='GPT in language models stands for Generative Pre-trained Transformer.')
gptmodel.addFit(prompt='What is the capital of Spain?', answer='The capital of Spain is Madrid.')
gptmodel.addFit(prompt='Tell me the name of the capital of Spain.', answer='The capital of Spain is Madrid.')
gptmodel.train(progress=False) # set False to the "progress" parameter if you want to hide the training progress (the default value is True)

answer = gptmodel.predict(prompt='What is the capital of Spain?')
print(answer)

```
Prediction response:
```bash
The capital of Spain is Madrid.
```
It is not recommended to train the model every time you want to infer an answer, as this will make the inference process slow and painful. To avoid this, we use the "saveModel" method to train the model only once and reuse the pre-trained model in future inferences. The "saveModel" method will save a file in GPT format in any directory of your choice.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# data creation for the training dataset
gptmodel.addFit(prompt='What does the acronym GPT mean in language models?', answer='GPT in language models stands for Generative Pre-trained Transformer.')
gptmodel.addFit(prompt='Tell me what is the meaning of GPT in language models.', answer='GPT in language models stands for Generative Pre-trained Transformer.')
gptmodel.addFit(prompt='What is the capital of Spain?', answer='The capital of Spain is Madrid.')
gptmodel.addFit(prompt='Tell me the name of the capital of Spain.', answer='The capital of Spain is Madrid.')
# training of model
gptmodel.train()
# saving the trained model, if no name is specified for the model file, the model will be saved with the name "model.gpt" in the local directory
gptmodel.saveModel()

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  17.9it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0283; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0875; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000031 - current precision is 0.1699; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000032 - current precision is 0.2325; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000033 - current precision is 0.3005; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000034 - current precision is 0.3450; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000035 - current precision is 0.4120; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000036 - current precision is 0.4631; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000037 - current precision is 0.4603; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000038 - current precision is 0.5211; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000039 - current precision is 0.5684; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000040 - current precision is 0.6030; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000041 - current precision is 0.5427; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000042 - current precision is 0.6152; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000043 - current precision is 0.6458; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000044 - current precision is 0.6456; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000045 - current precision is 0.7218; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000046 - current precision is 0.7154; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000047 - current precision is 0.6967; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000048 - current precision is 0.7382; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000049 - current precision is 0.7145; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.1it/s]
Backpropagation epoch: 0000000050 - current precision is 0.7290; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000051 - current precision is 0.7859; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000052 - current precision is 0.7316; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000053 - current precision is 0.7666; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000054 - current precision is 0.7638; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000055 - current precision is 0.7964; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000056 - current precision is 0.7907; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000057 - current precision is 0.7540; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000058 - current precision is 0.8176; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000059 - current precision is 0.7576; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000060 - current precision is 0.7844; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000061 - current precision is 0.7531; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000062 - current precision is 0.7972; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000063 - current precision is 0.7963; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000064 - current precision is 0.7934; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000065 - current precision is 0.8080; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000066 - current precision is 0.8438; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.6it/s]
Backpropagation epoch: 0000000067 - current precision is 0.8092; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000068 - current precision is 0.7846; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000069 - current precision is 0.8207; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000070 - current precision is 0.8446; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000071 - current precision is 0.8180; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000072 - current precision is 0.8227; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000073 - current precision is 0.8412; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000074 - current precision is 0.8510; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000075 - current precision is 0.8168; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000076 - current precision is 0.8321; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000077 - current precision is 0.8715; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000078 - current precision is 0.8669; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000079 - current precision is 0.8821; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000080 - current precision is 0.8860; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000081 - current precision is 0.8985; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000082 - current precision is 0.8843; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000083 - current precision is 0.8249; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.4it/s]
Backpropagation epoch: 0000000084 - current precision is 0.8355; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000085 - current precision is 0.8973; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000086 - current precision is 0.7562; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000087 - current precision is 0.7851; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000088 - current precision is 0.8791; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000089 - current precision is 0.8691; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000090 - current precision is 0.8734; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000091 - current precision is 0.8843; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000092 - current precision is 0.8804; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000093 - current precision is 0.8981; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000094 - current precision is 0.8980; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000095 - current precision is 0.8949; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000096 - current precision is 0.9232; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 37.1kit/s]

```
Use the method named 'loadModel' if you want to load a pre-trained model for a faster inference.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# there is no need to waste time on retraining if you are loading a model that has already been pre-trained and saved previously
gptmodel.loadModel() # if no model file name is specified, the method will look in the local directory for a model with default name, in this case with the name "model.gpt"
# with the pre-loaded template, simply ask any question about the data contained in this template
answer = gptmodel.predict(prompt='what is the capital of spain?')
print(answer)

```
You can repeat the execution of the same inference as many times as you want, until you get a satisfactory answer.
```bash
The capital of Spain is Madrid.
```
Now see how to define a name for your saved model.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()

gptmodel.addFit(prompt='What does the acronym GPT mean in language models?', answer='GPT in language models stands for Generative Pre-trained Transformer.')
gptmodel.addFit(prompt='Tell me what is the meaning of GPT in language models.', answer='GPT in language models stands for Generative Pre-trained Transformer.')
gptmodel.addFit(prompt='What is the capital of Spain?', answer='The capital of Spain is Madrid.')
gptmodel.addFit(prompt='Tell me the name of the capital of Spain.', answer='The capital of Spain is Madrid.')

gptmodel.train()
# use the "model_path" parameter if you want to specify a path and/or name for your model
gptmodel.saveModel(model_path='my_model.gpt') # the ".gpt" extension is optional as it will be automatically added to the end of the file name

```
Note that by default, training will seek an precision of at least 0.9 (90%), i.e. there must be a maximum loss of 10%.
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  17.1it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0492; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000030 - current precision is 0.1132; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000031 - current precision is 0.2063; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000032 - current precision is 0.2559; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000033 - current precision is 0.3057; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000034 - current precision is 0.3559; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000035 - current precision is 0.4133; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000036 - current precision is 0.4535; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000037 - current precision is 0.5021; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000038 - current precision is 0.5373; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000039 - current precision is 0.5619; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000040 - current precision is 0.5499; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000041 - current precision is 0.5979; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000042 - current precision is 0.5785; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000043 - current precision is 0.6225; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000044 - current precision is 0.6364; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000045 - current precision is 0.6767; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000046 - current precision is 0.6706; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000047 - current precision is 0.6884; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000048 - current precision is 0.7188; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000049 - current precision is 0.6907; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000050 - current precision is 0.7327; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000051 - current precision is 0.7220; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000052 - current precision is 0.7541; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000053 - current precision is 0.7334; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000054 - current precision is 0.7532; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000055 - current precision is 0.7714; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000056 - current precision is 0.7928; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000057 - current precision is 0.7929; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000058 - current precision is 0.8276; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000059 - current precision is 0.8111; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000060 - current precision is 0.7478; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000061 - current precision is 0.8167; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000062 - current precision is 0.7089; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000063 - current precision is 0.7122; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000064 - current precision is 0.8267; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000065 - current precision is 0.7944; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000066 - current precision is 0.8348; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000067 - current precision is 0.8335; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000068 - current precision is 0.7601; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000069 - current precision is 0.7915; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000070 - current precision is 0.8493; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000071 - current precision is 0.8058; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000072 - current precision is 0.7805; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000073 - current precision is 0.8324; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000074 - current precision is 0.8366; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000075 - current precision is 0.8142; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000076 - current precision is 0.8583; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000077 - current precision is 0.8827; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000078 - current precision is 0.8466; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000079 - current precision is 0.8911; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000080 - current precision is 0.8636; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000081 - current precision is 0.8697; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000082 - current precision is 0.8807; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000083 - current precision is 0.8749; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000084 - current precision is 0.8739; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000085 - current precision is 0.9105; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 36.8kit/s]

```
Now just use the loading method to load the model that was pre-trained earlier.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# use the "model_path" parameter if you want to specify a path and/or name for your model
gptmodel.loadModel(model_path='my_model.gpt') # the ".gpt" extension is optional as it will be automatically added to the end of the file name

answer = gptmodel.predict(prompt='what is the capital of spain?')
print(answer)

```
```bash
The capital of Spain is Madrid.
```
You can also specify a path to save your template. If the specified path does not exist, it will be automatically created.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()

gptmodel.addFit(prompt='What does the acronym GPT mean in language models?', answer='GPT in language models stands for Generative Pre-trained Transformer.')
gptmodel.addFit(prompt='Tell me what is the meaning of GPT in language models.', answer='GPT in language models stands for Generative Pre-trained Transformer.')
gptmodel.addFit(prompt='What is the capital of Spain?', answer='The capital of Spain is Madrid.')
gptmodel.addFit(prompt='Tell me the name of the capital of Spain.', answer='The capital of Spain is Madrid.')

gptmodel.train()

gptmodel.saveModel(model_path='./models/my_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  17.9it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0407; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000030 - current precision is 0.1057; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000031 - current precision is 0.1635; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000032 - current precision is 0.2295; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000033 - current precision is 0.2910; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000034 - current precision is 0.3574; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000035 - current precision is 0.3811; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000036 - current precision is 0.4315; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000037 - current precision is 0.4777; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000038 - current precision is 0.5258; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000039 - current precision is 0.5476; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000040 - current precision is 0.5700; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000041 - current precision is 0.5727; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000042 - current precision is 0.6256; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000043 - current precision is 0.5909; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000044 - current precision is 0.6327; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000045 - current precision is 0.6671; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000046 - current precision is 0.6716; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000047 - current precision is 0.7029; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000048 - current precision is 0.6911; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000049 - current precision is 0.6983; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000050 - current precision is 0.7157; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000051 - current precision is 0.6853; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000052 - current precision is 0.7200; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000053 - current precision is 0.7264; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000054 - current precision is 0.7670; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000055 - current precision is 0.7735; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000056 - current precision is 0.7494; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000057 - current precision is 0.7601; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000058 - current precision is 0.7812; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000059 - current precision is 0.7629; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000060 - current precision is 0.8097; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000061 - current precision is 0.8142; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000062 - current precision is 0.7881; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000063 - current precision is 0.8211; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.5it/s]
Backpropagation epoch: 0000000064 - current precision is 0.7930; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000065 - current precision is 0.8361; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.6it/s]
Backpropagation epoch: 0000000066 - current precision is 0.7767; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.6it/s]
Backpropagation epoch: 0000000067 - current precision is 0.7900; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.2it/s]
Backpropagation epoch: 0000000068 - current precision is 0.8520; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000069 - current precision is 0.7965; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000070 - current precision is 0.7728; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000071 - current precision is 0.7930; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000072 - current precision is 0.8438; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000073 - current precision is 0.8188; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000074 - current precision is 0.8557; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000075 - current precision is 0.8787; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000076 - current precision is 0.8571; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000077 - current precision is 0.8701; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000078 - current precision is 0.8905; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000079 - current precision is 0.8862; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000080 - current precision is 0.8899; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000081 - current precision is 0.8745; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000082 - current precision is 0.8923; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000083 - current precision is 0.8759; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000084 - current precision is 0.9353; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 39.9kit/s]

```
After just specify the same path in the "loadModel" method.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()

gptmodel.loadModel(model_path='./models/my_model.gpt')

answer = gptmodel.predict(prompt='what is the capital of spain?')
print(answer)

```
```bash
The capital of Spain is Madrid.
```
Instead of calling the "addFit" method multiple times, you can just call the "train" method once, passing a string with all the training data.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# string variable containing the text that will be used in training the model
string = '''
What does the acronym GPT mean in language models?
GPT in language models stands for Generative Pre-trained Transformer.

Tell me what is the meaning of GPT in language models.
GPT in language models stands for Generative Pre-trained Transformer.

What is the capital of Spain?
The capital of Spain is Madrid.

Tell me the name of the capital of Spain.
The capital of Spain is Madrid.
'''
# use the "string" name parameter contained in the "train" method to train the model with all the data at once
gptmodel.train(string=string)
# a model file will be created in the "models" directory with the name "gpt_model.gpt"
gptmodel.saveModel(model_path='./models/gpt_model') # note that the ".gpt" extension is optional

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  16.7it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.6it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0282; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0903; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000030 - current precision is 0.1696; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000031 - current precision is 0.2380; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000032 - current precision is 0.2866; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000033 - current precision is 0.3251; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000034 - current precision is 0.4063; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000035 - current precision is 0.4324; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000036 - current precision is 0.4781; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000037 - current precision is 0.5207; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.3it/s]
Backpropagation epoch: 0000000038 - current precision is 0.5470; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000039 - current precision is 0.5781; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000040 - current precision is 0.6142; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000041 - current precision is 0.6444; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000042 - current precision is 0.6591; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000043 - current precision is 0.6534; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000044 - current precision is 0.6691; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000045 - current precision is 0.6603; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000046 - current precision is 0.7223; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000047 - current precision is 0.7201; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000048 - current precision is 0.6825; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000049 - current precision is 0.7367; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000050 - current precision is 0.7162; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000051 - current precision is 0.7140; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000052 - current precision is 0.7707; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000053 - current precision is 0.7718; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000054 - current precision is 0.7651; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000055 - current precision is 0.7720; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000056 - current precision is 0.7852; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000057 - current precision is 0.7881; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.1it/s]
Backpropagation epoch: 0000000058 - current precision is 0.7805; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000059 - current precision is 0.8073; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000060 - current precision is 0.7566; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000061 - current precision is 0.8096; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000062 - current precision is 0.7663; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000063 - current precision is 0.8169; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000064 - current precision is 0.8006; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000065 - current precision is 0.7781; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000066 - current precision is 0.7970; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000067 - current precision is 0.8147; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000068 - current precision is 0.7989; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000069 - current precision is 0.8462; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000070 - current precision is 0.8380; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000071 - current precision is 0.8346; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000072 - current precision is 0.8351; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000073 - current precision is 0.8688; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000074 - current precision is 0.8460; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000075 - current precision is 0.8486; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000076 - current precision is 0.8641; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000077 - current precision is 0.8569; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000078 - current precision is 0.8762; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000079 - current precision is 0.8101; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000080 - current precision is 0.8468; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000081 - current precision is 0.8804; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000082 - current precision is 0.8705; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000083 - current precision is 0.8871; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000084 - current precision is 0.8940; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000085 - current precision is 0.9005; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 38.8kit/s]

```
Now loading the previously pretrained model.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# a file named "gpt_model.gpt" will be loaded from within the "models" directory
gptmodel.loadModel(model_path='./models/gpt_model') # note that the ".gpt" extension is optional

answer = gptmodel.predict(prompt='What is the capital of Spain?')
print(answer)

```
Note that there will be unwanted repetitions in the response returned by the inference. To avoid this we must define a response end tag in our training data.
```bash
The capital of Spain is Madrid.                                                                                                                                 
Tell me the name of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
Tell me the name of Spain.
The capital of Spain.
Tell me Spain is Madrid.
Tell me the name of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.

Tell me the name of Spain is Madrid.
The capital of Spain.
The capital of Spain.

The capital of Spain.
The capital of Spain.
The capital of Spain.

The capital of Spain.
The capital of Spain.
The capital of Spain is Madrid.
The capital of Spain.
The capital of Spain is Madrid.
The capital of Spain.
Tell me the name of Spain.
The capital of Spain.
The capital of Spain is Madrid.
The capital of Spain.
The capital of Spain is Madrid.
The capital of Spain is Madrid.
The capital of Spain.
The capital of Spain is Madrid.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain is Madrid.
The capital of Spain.


The capital of Spain.
Tell me the name of Spain is Madrid.
Tell me the name of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain is Madrid.
The capital of Spain.
The capital of Spain.
Tell me the name of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.
The capital of Spain.

The capital of Spain.
The capital of Spain.
```
After each answer, we will define any tag that delimits how far the answer should consult the tokens.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# we chose the tag "<|end|>" to delimiter the end of the answers, but you can create any tag of your choice
# always repeat the same tag after each of the answers
# note: insert the tags at the far right end of each response
string = '''
What does the acronym GPT mean in language models?
GPT in language models stands for Generative Pre-trained Transformer.<|end|>

Tell me what is the meaning of GPT in language models.
GPT in language models stands for Generative Pre-trained Transformer.<|end|>

What is the capital of Spain?
The capital of Spain is Madrid.<|end|>

Tell me the name of the capital of Spain.
The capital of Spain is Madrid.<|end|>
'''
# now in training use the "end_tag" parameter so that the method knows which end tag was used in the dataset
# "end_tag" should receive a string with the same tag applied to the dataset above
gptmodel.train(string=string, end_tag='<|end|>')
# set False to the "progress" parameter if you do not want to view the saving progress (the default value is True)
gptmodel.saveModel(model_path='./models/gpt_model', progress=False) # the previous template with the same name will be overwritten

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  17.8it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.3it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.5it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.3it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.1it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.4it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.4it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.4it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.3it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.3it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.3it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0088; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0773; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000031 - current precision is 0.1395; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000032 - current precision is 0.2023; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000033 - current precision is 0.2765; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000034 - current precision is 0.3117; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000035 - current precision is 0.3813; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000036 - current precision is 0.4142; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000037 - current precision is 0.4728; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000038 - current precision is 0.4985; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000039 - current precision is 0.5181; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000040 - current precision is 0.5760; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000041 - current precision is 0.5670; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000042 - current precision is 0.6114; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000043 - current precision is 0.6141; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000044 - current precision is 0.6621; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000045 - current precision is 0.6978; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000046 - current precision is 0.7053; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000047 - current precision is 0.7315; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000048 - current precision is 0.7341; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000049 - current precision is 0.7380; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000050 - current precision is 0.7627; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000051 - current precision is 0.7805; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000052 - current precision is 0.7987; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000053 - current precision is 0.7466; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000054 - current precision is 0.7603; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000055 - current precision is 0.7826; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000056 - current precision is 0.6715; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000057 - current precision is 0.6981; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000058 - current precision is 0.7791; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000059 - current precision is 0.7608; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000060 - current precision is 0.8010; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.4it/s]
Backpropagation epoch: 0000000061 - current precision is 0.8179; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000062 - current precision is 0.7921; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000063 - current precision is 0.8113; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000064 - current precision is 0.8090; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000065 - current precision is 0.8360; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000066 - current precision is 0.8282; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000067 - current precision is 0.8484; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000068 - current precision is 0.8509; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000069 - current precision is 0.8447; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000070 - current precision is 0.8029; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000071 - current precision is 0.8688; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000072 - current precision is 0.7863; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000073 - current precision is 0.8418; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000074 - current precision is 0.8846; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000075 - current precision is 0.8670; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000076 - current precision is 0.8572; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000077 - current precision is 0.8683; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000078 - current precision is 0.8615; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000079 - current precision is 0.8853; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000080 - current precision is 0.8594; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000081 - current precision is 0.8771; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000082 - current precision is 0.9103; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 39.9kit/s]

```
Now perform a new inference by loading the updated model.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# set False to the "progress" parameter if you do not want to view the loading progress (the default value is True)
gptmodel.loadModel(model_path='./models/gpt_model', progress=False)
answer = gptmodel.predict(prompt='What is the capital of Spain?')
print(answer)

```
```bash
The capital of Spain is Madrid.
```
If you prefer, you don't need to insert blank lines between one pair and another.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# the blank line between one pair and another is not mandatory, but it is recommended
string = '''
What does the acronym GPT mean in language models?
GPT in language models stands for Generative Pre-trained Transformer.<|end|>
Tell me what is the meaning of GPT in language models.
GPT in language models stands for Generative Pre-trained Transformer.<|end|>
What is the capital of Spain?
The capital of Spain is Madrid.<|end|>
Tell me the name of the capital of Spain.
The capital of Spain is Madrid.<|end|>
'''

gptmodel.train(string=string, end_tag='<|end|>')
# it is not necessary to use "./" to refer to the root directory, but it is recommended
gptmodel.saveModel(model_path='models/gpt_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  17.5it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.7it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0042; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0697; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000030 - current precision is 0.1360; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000031 - current precision is 0.2029; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000032 - current precision is 0.2764; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000033 - current precision is 0.3145; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000034 - current precision is 0.3730; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000035 - current precision is 0.4348; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000036 - current precision is 0.4817; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000037 - current precision is 0.5384; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000038 - current precision is 0.5734; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000039 - current precision is 0.6025; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000040 - current precision is 0.6304; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000041 - current precision is 0.6690; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000042 - current precision is 0.6777; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000043 - current precision is 0.6915; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000044 - current precision is 0.7186; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000045 - current precision is 0.7320; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000046 - current precision is 0.7529; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000047 - current precision is 0.7560; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000048 - current precision is 0.7671; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000049 - current precision is 0.7785; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000050 - current precision is 0.8046; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000051 - current precision is 0.7624; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000052 - current precision is 0.8075; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000053 - current precision is 0.7298; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000054 - current precision is 0.6975; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000055 - current precision is 0.7753; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000056 - current precision is 0.8139; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000057 - current precision is 0.7722; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000058 - current precision is 0.7809; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.3it/s]
Backpropagation epoch: 0000000059 - current precision is 0.7862; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.4it/s]
Backpropagation epoch: 0000000060 - current precision is 0.8269; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000061 - current precision is 0.8404; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000062 - current precision is 0.8270; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000063 - current precision is 0.8210; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000064 - current precision is 0.8268; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000065 - current precision is 0.8379; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000066 - current precision is 0.8370; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000067 - current precision is 0.8588; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.1it/s]
Backpropagation epoch: 0000000068 - current precision is 0.8725; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000069 - current precision is 0.8618; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000070 - current precision is 0.8664; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000071 - current precision is 0.8693; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000072 - current precision is 0.8201; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000073 - current precision is 0.8899; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000074 - current precision is 0.8811; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000075 - current precision is 0.8737; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000076 - current precision is 0.9010; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 41.5kit/s]

```
For loading the model, the structure is the same as we already know.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# it is not necessary to use "./" to refer to the root directory, but it is recommended
gptmodel.loadModel(model_path='models/gpt_model.gpt')
answer = gptmodel.predict(prompt='Tell me what is the meaning of GPT in language models.')
print(answer)

```
```bash
GPT in language models stands for Generative Pre-trained Transformer.
```
The best, most reliable and most recommended way to train your model is through a structured file in JSON format.
The vast majority of widely known foundation language models are trained with very well-structured files in JSON format. The training JSON should have a primary key named "data" containing an array with the training data arranged as objects containing a key named "input" for each sample of input/question/instruction and a key named "output" for each desired response. This prevents misinterpretation of the model and increases the degree of accuracy and reliability in the responses.
Example JSON (dataset.json file contents):
```json
{
	"data": [
				{
					"input": "Hello! Who are you?",
					"output": "Hello! I am Sapiens Chat, an AI model created by Sapiens Technology."
				},
				{
					"input": "Who discovered Brazil?",
					"output": "Brazil was discovered by Portuguese navigators led by Pedro Alvares Cabral in 1500."
				},
				{
					"input": "What is the main language spoken in Spain?",
					"output": "The main language spoken in Spain is Spanish."
				},
				{
					"input": "What is the capital of Portugal?",
					"output": "The capital of Portugal is Lisbon."
				},
				{
					"input": "How much is 2 + 2?",
					"output": "The sum of 2 + 2 in mathematics is 4."
				},
				{
					"input": "What is five minus three?",
					"output": "The result of five minus three is two."
				},
				{
					"input": "What is your name?",
					"output": "My name is Sapiens Chat."
				}
	]
}
```
To train your model with a JSON or TXT file, use the parameter named "dataset_path", which should receive a string with the path to your training dataset.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# pass the path of your json or txt file to the value of the "dataset_path" argument
# txt files do not require structuring, although it is highly recommended to use finalization tags in the content of txt files and then define this same tag in the value of "end_tag"
# avoid using files in txt format and always give preference to files structured in json format
gptmodel.train(dataset_path='dataset.json')
# it is not necessary to use "./" to refer to the root directory, but it is recommended
# remember that the ".gpt" extension is also optional, although recommended
gptmodel.saveModel(model_path='models/my_model') # we are using the name "my_model" as an example, but you can give your model any name you want

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  17.9it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.6it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.3it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.3it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000031 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000033 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000034 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0073; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000036 - current precision is 0.0722; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000037 - current precision is 0.1163; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000038 - current precision is 0.1707; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000039 - current precision is 0.2189; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000040 - current precision is 0.2552; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000041 - current precision is 0.2956; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000042 - current precision is 0.3404; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000043 - current precision is 0.3552; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000044 - current precision is 0.4164; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000045 - current precision is 0.4282; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000046 - current precision is 0.4392; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000047 - current precision is 0.4669; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000048 - current precision is 0.4841; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000049 - current precision is 0.4998; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000050 - current precision is 0.5336; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000051 - current precision is 0.5436; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000052 - current precision is 0.5572; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000053 - current precision is 0.5727; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000054 - current precision is 0.5822; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000055 - current precision is 0.5933; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000056 - current precision is 0.6090; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000057 - current precision is 0.5865; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000058 - current precision is 0.6315; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000059 - current precision is 0.6287; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000060 - current precision is 0.6091; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000061 - current precision is 0.6494; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000062 - current precision is 0.6657; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000063 - current precision is 0.6681; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000064 - current precision is 0.6635; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000065 - current precision is 0.6715; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000066 - current precision is 0.6798; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000067 - current precision is 0.6920; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000068 - current precision is 0.7086; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000069 - current precision is 0.6548; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000070 - current precision is 0.7216; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000071 - current precision is 0.6209; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000072 - current precision is 0.7131; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000073 - current precision is 0.6880; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000074 - current precision is 0.6890; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000075 - current precision is 0.7031; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000076 - current precision is 0.7263; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000077 - current precision is 0.7626; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000078 - current precision is 0.6642; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000079 - current precision is 0.7332; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000080 - current precision is 0.6602; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.6it/s]
Backpropagation epoch: 0000000081 - current precision is 0.6926; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000082 - current precision is 0.6583; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000083 - current precision is 0.6691; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000084 - current precision is 0.7652; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000085 - current precision is 0.7345; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000086 - current precision is 0.7140; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000087 - current precision is 0.7606; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000088 - current precision is 0.7480; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000089 - current precision is 0.7477; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000090 - current precision is 0.7687; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000091 - current precision is 0.7851; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000092 - current precision is 0.7999; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000093 - current precision is 0.7793; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000094 - current precision is 0.8141; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000095 - current precision is 0.7953; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000096 - current precision is 0.8092; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000097 - current precision is 0.7861; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000098 - current precision is 0.8532; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000099 - current precision is 0.8344; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000100 - current precision is 0.8616; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000101 - current precision is 0.8224; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000102 - current precision is 0.8581; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000103 - current precision is 0.8877; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000104 - current precision is 0.8671; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000105 - current precision is 0.8587; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000106 - current precision is 0.8085; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000107 - current precision is 0.8592; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000108 - current precision is 0.8726; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000109 - current precision is 0.8627; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000110 - current precision is 0.8873; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000111 - current precision is 0.8697; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000112 - current precision is 0.8780; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000113 - current precision is 0.8541; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000114 - current precision is 0.8825; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000115 - current precision is 0.8221; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000116 - current precision is 0.8446; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000117 - current precision is 0.8019; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000118 - current precision is 0.8337; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000119 - current precision is 0.8720; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000120 - current precision is 0.8606; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000121 - current precision is 0.8506; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000122 - current precision is 0.8763; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000123 - current precision is 0.9031; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 32.3kit/s]

```
Now we will test our trained model with the JSON file.
If your model's responses are very unsatisfactory, you should increase the size of your dataset.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# it is not necessary to use "./" to refer to the root directory, but it is recommended
# remember that the ".gpt" extension is also optional, although recommended
gptmodel.loadModel(model_path='models/my_model')
answer = gptmodel.predict(prompt='Hello! Who are you?')
print(answer)

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
Although not recommended, you can also train your model with a raw, unstructured txt data file.
Example TXT (dataset.txt file contents):
```txt
What is Artificial Intelligence (AI)?
Artificial Intelligence is a field of computer science that develops systems capable of performing tasks that would normally require human intelligence, such as learning, reasoning, perception, and decision-making.

What is the difference between weak AI and strong AI?
Weak AI (or narrow AI) is designed for specific tasks, such as virtual assistants and recommendation systems. Strong AI, on the other hand, would have self-awareness and the ability to understand and reason about any subject like a human being.

What is Machine Learning and how is it related to AI?
Machine Learning is a subfield of AI that teaches machines to learn patterns from data without explicit programming. It allows systems to improve their performance over time with experience.

What are the main types of machine learning?
The three main types are:
- **Supervised Learning**: The model learns from labeled data.
- **Unsupervised Learning**: The model identifies patterns without labels.
- **Reinforcement Learning**: The model learns through trial and error, receiving rewards or penalties.

What are artificial neural networks?
They are models inspired by the functioning of the human brain, composed of layers of artificial neurons that process information and adjust their weights to recognize patterns and make decisions.

What are the risks of Artificial Intelligence?
Some risks include algorithmic bias, job loss due to automation, misuse in surveillance or warfare, and the possibility of superintelligence beyond human control.

How is AI used in everyday life?
AI is present in virtual assistants (Siri, Alexa), recommendation systems (Netflix, Spotify), facial recognition, autonomous cars, medical diagnostics, chatbots, and much more.

What is natural language processing (NLP)?
It is a field of AI that enables machines to understand, interpret, and generate human language, used in machine translation, chatbots, and voice assistants.

Can AI replace humans in the workforce?
AI can automate repetitive and analytical tasks, but it is unlikely to fully replace humans in creative, emotional, and critical thinking jobs.  

What is a generative AI model?
It is a type of AI that can create new content, such as images, text, and music, based on patterns learned from large amounts of data. Examples include ChatGPT and DALL·E.

```
```python
from gptmodel import GPTModel
gptmodel = GPTModel()

gptmodel.train(dataset_path='dataset.txt')
gptmodel.saveModel(model_path='./models/my_model.gpt')

```
If necessary, retrain your model until your inferences become satisfactory.
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  16.8it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000031 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000033 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000034 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000036 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.3it/s]
Backpropagation epoch: 0000000037 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000038 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000039 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000040 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000041 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000042 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000043 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000044 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000045 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000046 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000047 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000048 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000049 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000050 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000051 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000052 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000053 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000054 - current precision is 0.0271; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000055 - current precision is 0.0511; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000056 - current precision is 0.0963; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000057 - current precision is 0.1087; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000058 - current precision is 0.1285; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000059 - current precision is 0.1486; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000060 - current precision is 0.1688; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000061 - current precision is 0.2057; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000062 - current precision is 0.2119; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000063 - current precision is 0.2365; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000064 - current precision is 0.2389; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000065 - current precision is 0.2637; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000066 - current precision is 0.2874; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000067 - current precision is 0.2940; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000068 - current precision is 0.3078; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000069 - current precision is 0.3276; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000070 - current precision is 0.3296; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000071 - current precision is 0.3587; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000072 - current precision is 0.3768; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000073 - current precision is 0.3916; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000074 - current precision is 0.3819; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000075 - current precision is 0.4203; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000076 - current precision is 0.3974; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000077 - current precision is 0.4349; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000078 - current precision is 0.4376; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000079 - current precision is 0.4504; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000080 - current precision is 0.4602; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000081 - current precision is 0.4370; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000082 - current precision is 0.4662; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000083 - current precision is 0.4419; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000084 - current precision is 0.4697; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000085 - current precision is 0.4845; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000086 - current precision is 0.4916; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000087 - current precision is 0.5235; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000088 - current precision is 0.5231; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000089 - current precision is 0.5100; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000090 - current precision is 0.5455; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000091 - current precision is 0.5396; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000092 - current precision is 0.5488; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000093 - current precision is 0.5560; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000094 - current precision is 0.5673; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000095 - current precision is 0.5898; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000096 - current precision is 0.5961; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000097 - current precision is 0.5892; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000098 - current precision is 0.6117; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000099 - current precision is 0.6268; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000100 - current precision is 0.6313; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000101 - current precision is 0.6373; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000102 - current precision is 0.6219; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000103 - current precision is 0.6225; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000104 - current precision is 0.6556; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000105 - current precision is 0.6541; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000106 - current precision is 0.6460; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000107 - current precision is 0.6716; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000108 - current precision is 0.6915; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000109 - current precision is 0.6897; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000110 - current precision is 0.6786; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.7it/s]
Backpropagation epoch: 0000000111 - current precision is 0.7229; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.1it/s]
Backpropagation epoch: 0000000112 - current precision is 0.6846; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.7it/s]
Backpropagation epoch: 0000000113 - current precision is 0.7106; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.2it/s]
Backpropagation epoch: 0000000114 - current precision is 0.7132; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.6it/s]
Backpropagation epoch: 0000000115 - current precision is 0.7223; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.7it/s]
Backpropagation epoch: 0000000116 - current precision is 0.7285; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000117 - current precision is 0.7543; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000118 - current precision is 0.7264; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.2it/s]
Backpropagation epoch: 0000000119 - current precision is 0.7263; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.1it/s]
Backpropagation epoch: 0000000120 - current precision is 0.7364; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.7it/s]
Backpropagation epoch: 0000000121 - current precision is 0.7397; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.0it/s]
Backpropagation epoch: 0000000122 - current precision is 0.7697; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.5it/s]
Backpropagation epoch: 0000000123 - current precision is 0.7445; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000124 - current precision is 0.7800; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000125 - current precision is 0.7713; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000126 - current precision is 0.7601; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000127 - current precision is 0.7943; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000128 - current precision is 0.7507; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000129 - current precision is 0.7904; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000130 - current precision is 0.7618; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000131 - current precision is 0.7959; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000132 - current precision is 0.8051; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000133 - current precision is 0.7874; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000134 - current precision is 0.8034; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000135 - current precision is 0.8024; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000136 - current precision is 0.8112; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000137 - current precision is 0.8249; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000138 - current precision is 0.7907; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000139 - current precision is 0.8113; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000140 - current precision is 0.8342; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000141 - current precision is 0.8315; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000142 - current precision is 0.8441; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000143 - current precision is 0.8315; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000144 - current precision is 0.8280; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000145 - current precision is 0.8499; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000146 - current precision is 0.8336; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000147 - current precision is 0.8551; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000148 - current precision is 0.8499; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000149 - current precision is 0.8505; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000150 - current precision is 0.8506; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000151 - current precision is 0.8645; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000152 - current precision is 0.8459; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000153 - current precision is 0.8640; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000154 - current precision is 0.8633; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000155 - current precision is 0.8621; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000156 - current precision is 0.8675; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000157 - current precision is 0.8681; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000158 - current precision is 0.8579; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000159 - current precision is 0.8685; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000160 - current precision is 0.8861; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000161 - current precision is 0.8896; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000162 - current precision is 0.8664; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000163 - current precision is 0.8966; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.0it/s]
Backpropagation epoch: 0000000164 - current precision is 0.8905; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000165 - current precision is 0.8918; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000166 - current precision is 0.9050; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 20.9kit/s]

```
Now we can run model inference.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# since "model_path" and "prompt" are the first parameters of their respective functions, their names can be hidden
gptmodel.loadModel('./models/my_model.gpt')
# use the "max_tokens" parameter of the prediction method to set an approximate limit for the total number of tokens in the response
# "max_tokens" must be an integer greater than zero
print(gptmodel.predict('What is Artificial Intelligence (AI)?', max_tokens=30))

```
```bash
Artificial Intelligence is a field of computer science that develops systems capable of performing tasks that would normally require human intelligence, such as learning, reasoning, perception, and decision-making.
```
To avoid using the "max_tokens" parameter, you can use finalization tags in the content of your TXT file.
Note: if the data sample is not too small, you can risk inserting the tags below the responses.
Example TXT (dataset.txt file contents):
```txt
What is Artificial Intelligence (AI)?
Artificial Intelligence is a field of computer science that develops systems capable of performing tasks that would normally require human intelligence, such as learning, reasoning, perception, and decision-making.
<|end|>
What is the difference between weak AI and strong AI?
Weak AI (or narrow AI) is designed for specific tasks, such as virtual assistants and recommendation systems. Strong AI, on the other hand, would have self-awareness and the ability to understand and reason about any subject like a human being.
<|end|>
What is Machine Learning and how is it related to AI?
Machine Learning is a subfield of AI that teaches machines to learn patterns from data without explicit programming. It allows systems to improve their performance over time with experience.
<|end|>
What are the main types of machine learning?
The three main types are:
- **Supervised Learning**: The model learns from labeled data.
- **Unsupervised Learning**: The model identifies patterns without labels.
- **Reinforcement Learning**: The model learns through trial and error, receiving rewards or penalties.
<|end|>
What are artificial neural networks?
They are models inspired by the functioning of the human brain, composed of layers of artificial neurons that process information and adjust their weights to recognize patterns and make decisions.
<|end|>
What are the risks of Artificial Intelligence?
Some risks include algorithmic bias, job loss due to automation, misuse in surveillance or warfare, and the possibility of superintelligence beyond human control.
<|end|>
How is AI used in everyday life?
AI is present in virtual assistants (Siri, Alexa), recommendation systems (Netflix, Spotify), facial recognition, autonomous cars, medical diagnostics, chatbots, and much more.
<|end|>
What is natural language processing (NLP)?
It is a field of AI that enables machines to understand, interpret, and generate human language, used in machine translation, chatbots, and voice assistants.
<|end|>
Can AI replace humans in the workforce?
AI can automate repetitive and analytical tasks, but it is unlikely to fully replace humans in creative, emotional, and critical thinking jobs.  
<|end|>
What is a generative AI model?
It is a type of AI that can create new content, such as images, text, and music, based on patterns learned from large amounts of data. Examples include ChatGPT and DALL·E.
<|end|>
```
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# since "dataset_path" and "model_path" are the first parameters of their respective functions, their names can be hidden
gptmodel.train('dataset.txt', end_tag='<|end|>')
gptmodel.saveModel('./models/my_model.gpt')

```
Final accuracies above 0.8 are considered highly accurate.
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.38it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.16it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.47it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.49it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.63it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.55it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.48it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.44it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.54it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.64it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.58it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.56it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.63it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.51it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.54it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.51it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.56it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.57it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.63it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.55it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.51it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.63it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.50it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.62it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0574; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.52it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0974; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.68it/s]
Backpropagation epoch: 0000000027 - current precision is 0.1711; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.68it/s]
Backpropagation epoch: 0000000028 - current precision is 0.2123; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.53it/s]
Backpropagation epoch: 0000000029 - current precision is 0.2627; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.54it/s]
Backpropagation epoch: 0000000030 - current precision is 0.2944; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.57it/s]
Backpropagation epoch: 0000000031 - current precision is 0.3177; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.61it/s]
Backpropagation epoch: 0000000032 - current precision is 0.3531; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.50it/s]
Backpropagation epoch: 0000000033 - current precision is 0.3893; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.46it/s]
Backpropagation epoch: 0000000034 - current precision is 0.4087; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.55it/s]
Backpropagation epoch: 0000000035 - current precision is 0.4309; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.60it/s]
Backpropagation epoch: 0000000036 - current precision is 0.4764; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.48it/s]
Backpropagation epoch: 0000000037 - current precision is 0.4979; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.62it/s]
Backpropagation epoch: 0000000038 - current precision is 0.5250; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.59it/s]
Backpropagation epoch: 0000000039 - current precision is 0.5289; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.53it/s]
Backpropagation epoch: 0000000040 - current precision is 0.5448; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.50it/s]
Backpropagation epoch: 0000000041 - current precision is 0.5705; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.62it/s]
Backpropagation epoch: 0000000042 - current precision is 0.5877; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.51it/s]
Backpropagation epoch: 0000000043 - current precision is 0.6036; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.52it/s]
Backpropagation epoch: 0000000044 - current precision is 0.6360; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.61it/s]
Backpropagation epoch: 0000000045 - current precision is 0.6439; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.54it/s]
Backpropagation epoch: 0000000046 - current precision is 0.6581; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.54it/s]
Backpropagation epoch: 0000000047 - current precision is 0.6705; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.51it/s]
Backpropagation epoch: 0000000048 - current precision is 0.6939; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.48it/s]
Backpropagation epoch: 0000000049 - current precision is 0.7131; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.50it/s]
Backpropagation epoch: 0000000050 - current precision is 0.7139; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.44it/s]
Backpropagation epoch: 0000000051 - current precision is 0.7275; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.47it/s]
Backpropagation epoch: 0000000052 - current precision is 0.7456; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.60it/s]
Backpropagation epoch: 0000000053 - current precision is 0.7572; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.46it/s]
Backpropagation epoch: 0000000054 - current precision is 0.7750; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.60it/s]
Backpropagation epoch: 0000000055 - current precision is 0.7759; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.65it/s]
Backpropagation epoch: 0000000056 - current precision is 0.7906; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.52it/s]
Backpropagation epoch: 0000000057 - current precision is 0.8013; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.59it/s]
Backpropagation epoch: 0000000058 - current precision is 0.8169; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.54it/s]
Backpropagation epoch: 0000000059 - current precision is 0.8233; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.53it/s]
Backpropagation epoch: 0000000060 - current precision is 0.8335; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.52it/s]
Backpropagation epoch: 0000000061 - current precision is 0.8431; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.47it/s]
Backpropagation epoch: 0000000062 - current precision is 0.8618; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.47it/s]
Backpropagation epoch: 0000000063 - current precision is 0.8641; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.52it/s]
Backpropagation epoch: 0000000064 - current precision is 0.8764; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.53it/s]
Backpropagation epoch: 0000000065 - current precision is 0.8813; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.47it/s]
Backpropagation epoch: 0000000066 - current precision is 0.8486; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.26it/s]
Backpropagation epoch: 0000000067 - current precision is 0.8875; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.53it/s]
Backpropagation epoch: 0000000068 - current precision is 0.8972; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:01<00:00,  1.48it/s]
Backpropagation epoch: 0000000069 - current precision is 0.9112; aiming for precision >= 0.9000 in training: 100%|██████████| 2.00/2.00 [00:00<00:00, 80.7kit/s]

```
Check below the final result of the inference of the created model.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
print(gptmodel.predict('What is Artificial Intelligence (AI)?'))

```
```bash
Artificial Intelligence is a field of computer science that develops systems capable of performing tasks that would normally require human intelligence, such as learning, reasoning, perception, and decision-making.

```
You can also supplement the data in your JSON or TXT dataset with the text from the string parameter.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
string = '''
What does the acronym GPT mean in language models?
GPT in language models stands for Generative Pre-trained Transformer.
<|end|>
Tell me what is the meaning of GPT in language models.
GPT in language models stands for Generative Pre-trained Transformer.
<|end|>
What is the capital of Spain?
The capital of Spain is Madrid.
<|end|>
Tell me the name of the capital of Spain.
The capital of Spain is Madrid.
<|end|>
'''
# to make training complete faster, we will decrease the maximum accuracy to 0.8
gptmodel.train('dataset.txt', string=string, precision=0.8, end_tag='<|end|>')
gptmodel.saveModel('./models/my_model.gpt')

```
The higher the final training accuracy, the lower the variation in responses to a given prompt. The lower the final training accuracy, the higher the variation in responses to a given prompt. Very high accuracies can also cause responses to be very faithful or even identical to the corresponding content in the training dataset. While lower accuracies cause responses to be more different from the corresponding content in the training dataset. The appropriate accuracy will depend on each use case and the specific needs of your project.
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:21<00:00,  4.36s/it]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:15<00:00,  3.00s/it]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:15<00:00,  3.16s/it]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:10<00:00,  2.19s/it]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:07<00:00,  1.52s/it]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:07<00:00,  1.46s/it]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:08<00:00,  1.72s/it]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:07<00:00,  1.49s/it]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:07<00:00,  1.48s/it]
Backpropagation epoch: 0000000010 - current precision is 0.0159; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:16<00:00,  3.30s/it]
Backpropagation epoch: 0000000011 - current precision is 0.1330; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:07<00:00,  1.52s/it]
Backpropagation epoch: 0000000012 - current precision is 0.2667; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:07<00:00,  1.51s/it]
Backpropagation epoch: 0000000013 - current precision is 0.3427; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:07<00:00,  1.50s/it]
Backpropagation epoch: 0000000014 - current precision is 0.4080; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:13<00:00,  2.75s/it]
Backpropagation epoch: 0000000015 - current precision is 0.4554; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:07<00:00,  1.58s/it]
Backpropagation epoch: 0000000016 - current precision is 0.5181; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:07<00:00,  1.44s/it]
Backpropagation epoch: 0000000017 - current precision is 0.5393; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:08<00:00,  1.62s/it]
Backpropagation epoch: 0000000018 - current precision is 0.6009; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:21<00:00,  4.31s/it]
Backpropagation epoch: 0000000019 - current precision is 0.5466; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:10<00:00,  2.06s/it]
Backpropagation epoch: 0000000020 - current precision is 0.6056; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:14<00:00,  2.94s/it]
Backpropagation epoch: 0000000021 - current precision is 0.6538; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:16<00:00,  3.33s/it]
Backpropagation epoch: 0000000022 - current precision is 0.6750; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:12<00:00,  2.53s/it]
Backpropagation epoch: 0000000023 - current precision is 0.7110; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:18<00:00,  3.79s/it]
Backpropagation epoch: 0000000024 - current precision is 0.7225; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:14<00:00,  2.91s/it]
Backpropagation epoch: 0000000025 - current precision is 0.7751; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:16<00:00,  3.27s/it]
Backpropagation epoch: 0000000026 - current precision is 0.8071; aiming for precision >= 0.8000 in training: 100%|██████████| 5.00/5.00 [00:00<00:00,  198kit/s]

```
Now you can submit related entries, both for the file and the training string value.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
print(gptmodel.predict('What is the difference between weak AI and strong AI?'))

```
Remember, if your answers are unsatisfactory you must add more data related to the wrong answers to your dataset.
```bash
Weak AI (or narrow AI) is designed for specific tasks, such as virtual assistants and recommendation systems. Strong AI, on the other hand, would have self-awareness and the ability to understand and reason about any subject like a human being.

```
The string parameter can also be combined with the contents of a JSON file.
Remember, datasets in JSON format provide much more reliable responses.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
dataset_path = './dataset.json'
string = '''
What does the acronym GPT mean in language models?
GPT in language models stands for Generative Pre-trained Transformer.
<|end|>
Tell me what is the meaning of GPT in language models.
GPT in language models stands for Generative Pre-trained Transformer.
<|end|>
What is the capital of Spain?
The capital of Spain is Madrid.
<|end|>
Tell me the name of the capital of Spain.
The capital of Spain is Madrid.
<|end|>
'''
gptmodel.train(dataset_path=dataset_path, string=string, end_tag='<|end|>')
gptmodel.saveModel('./models/my_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.1it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.5it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000031 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000033 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000034 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000036 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000037 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000038 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000039 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000040 - current precision is 0.0727; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000041 - current precision is 0.1069; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000042 - current precision is 0.1588; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000043 - current precision is 0.1994; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000044 - current precision is 0.2446; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000045 - current precision is 0.2599; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000046 - current precision is 0.2738; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000047 - current precision is 0.3178; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000048 - current precision is 0.3428; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000049 - current precision is 0.3702; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000050 - current precision is 0.3990; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000051 - current precision is 0.4037; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000052 - current precision is 0.4322; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000053 - current precision is 0.4561; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000054 - current precision is 0.4634; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000055 - current precision is 0.4794; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000056 - current precision is 0.4960; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000057 - current precision is 0.4886; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000058 - current precision is 0.4783; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000059 - current precision is 0.5300; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000060 - current precision is 0.4806; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000061 - current precision is 0.5065; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000062 - current precision is 0.5342; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000063 - current precision is 0.5424; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000064 - current precision is 0.5810; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000065 - current precision is 0.5573; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000066 - current precision is 0.5722; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000067 - current precision is 0.6033; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000068 - current precision is 0.5915; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000069 - current precision is 0.6211; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000070 - current precision is 0.6168; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000071 - current precision is 0.5245; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000072 - current precision is 0.5735; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000073 - current precision is 0.6465; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000074 - current precision is 0.6325; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000075 - current precision is 0.6574; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000076 - current precision is 0.6415; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000077 - current precision is 0.6687; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000078 - current precision is 0.6506; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000079 - current precision is 0.6539; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000080 - current precision is 0.6243; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000081 - current precision is 0.6634; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000082 - current precision is 0.6044; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000083 - current precision is 0.6400; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000084 - current precision is 0.6680; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000085 - current precision is 0.6725; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000086 - current precision is 0.6968; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000087 - current precision is 0.6894; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000088 - current precision is 0.6701; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000089 - current precision is 0.7150; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000090 - current precision is 0.7038; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000091 - current precision is 0.7012; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000092 - current precision is 0.7562; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000093 - current precision is 0.7266; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000094 - current precision is 0.7323; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000095 - current precision is 0.7620; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000096 - current precision is 0.7769; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000097 - current precision is 0.7712; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000098 - current precision is 0.7763; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000099 - current precision is 0.7662; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000100 - current precision is 0.7658; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000101 - current precision is 0.7440; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000102 - current precision is 0.7331; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000103 - current precision is 0.7865; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000104 - current precision is 0.6710; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000105 - current precision is 0.7222; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000106 - current precision is 0.7966; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000107 - current precision is 0.7508; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000108 - current precision is 0.7432; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000109 - current precision is 0.7553; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000110 - current precision is 0.7752; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000111 - current precision is 0.7906; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000112 - current precision is 0.8086; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000113 - current precision is 0.7629; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000114 - current precision is 0.7838; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000115 - current precision is 0.7996; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000116 - current precision is 0.7999; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000117 - current precision is 0.8215; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000118 - current precision is 0.8448; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000119 - current precision is 0.8417; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000120 - current precision is 0.8256; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000121 - current precision is 0.8516; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000122 - current precision is 0.8332; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000123 - current precision is 0.8498; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000124 - current precision is 0.8679; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000125 - current precision is 0.8462; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000126 - current precision is 0.8488; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000127 - current precision is 0.8940; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000128 - current precision is 0.8717; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000129 - current precision is 0.8242; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000130 - current precision is 0.8829; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000131 - current precision is 0.9046; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 24.4kit/s]

```
Check out an inference test below with the model created above.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
print(gptmodel.predict('Hello! Who are you?'))
# note that the question of "prompt" does not need to be identical to the training question
print(gptmodel.predict('I want to know the name of the capital of Spain.'))

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.                                                                                            
The capital of Spain is Madrid.

```
To fetch one token at a time while each token is generated in real time, set the value of the "stream" parameter to True.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
# use the "stream" parameter to True if you want to view the token-by-token response as tokens are generated in real time (the default value is False)
generator = gptmodel.predict('Hello! Who are you?', stream=True) # return each token one at a time
for token in generator: # loops through each returned token
    print(token, end='', flush=True) # displays one token in front of another with no space between them
print() # continues terminal execution on the next line

```
Final sequence generated token by token.
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
Control the temperature of your responses with the "temperature" parameter.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
# with the "temperature" parameter you will make the responses more different from each other as the value approaches 1, as well as being able to decrease the variability between one response and another as the value approaches 0 (default value is 0.5 for 50% variation)
# if the temperature is equal to zero, the model will output a deterministic result, always selecting the token with the highest possible probability.
generator = gptmodel.predict('Tell me the name of the capital of Spain.', temperature=0.7, stream=True) # in this case we are defining a variability in the responses of 70%, but this will not always return more varied responses if your final precision of training was very high
for token in generator: print(token, end='', flush=True)
print()

```
```bash
The name is Madrid.

```
With the "top_k" parameter you define an integer for the maximum number of next most likely tokens to be considered in the random selection of the prediction. If "top_k" is equal to zero, all tokens will be considered and any of them will be randomly returned to the right of the previous token. If "top_k" is equal to 1, only a single token will be considered that has the highest probability among all possible tokens, eliminating the random selection. If for example "top_k" is equal to 50, the 50 most likely tokens will be selected and among these 50, one of them will be randomly selected to complete the response.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
# with "top_k" equal to 50, the response will be completed token by token, randomly choosing the next token among the 50 most likely
# "top_k" must receive an integer number greater than or equal to zero
generator = gptmodel.predict('Hello! Who are you?', top_k=50, stream=True) # top_k=50 considers only the 50 most likely tokens, the default value for "top_k" is 0
for token in generator: print(token, end='', flush=True)
print()

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
With the "top_p" parameter you consider only the most likely tokens that have a probability greater than or equal to the value of "top_p" and select one of these tokens randomly to complete your response token by token. If, for example, "top_p" is equal to zero, no token will be selected, since probabilities equal to zero will always be disregarded. If, for example, "top_p" is equal to 1 with a temperature equal to 0, only the most likely token will be considered, eliminating the randomness in the choice. However, if, for example, "top_p" is equal to 0.9, all tokens with a probability greater than or equal to 90% will be considered and one of these considered tokens will be returned randomly.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
# with "top_p" equal to 0.9, the response will be completed token by token, randomly choosing the next token among all tokens with probability greater than or equal to 0.9
# "top_p" must receive a floating number between 0 and 1
# note: if "top_p" is equal to 1 (100%) all tokens will be considered and randomness will be at the discretion of the temperature
generator = gptmodel.predict('Hello! Who are you?', top_p=0.9, stream=True) # top_p=0.9 considers only tokens with probability greater than or equal to 90%, the default value for "top_p" is 1.0
for token in generator: print(token, end='', flush=True)
print()

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
Check out an example of a call below with all the prediction parameters and their default values.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')

prompt = 'Hello! Who are you?' # input prompt for prediction
# calling the "predict" function with all its parameters and their default values
# note: the default value of the "prompt" parameter is an empty string
answer = gptmodel.predict(prompt=prompt, max_tokens=500, temperature=0.5, top_k=0, top_p=1.0, stream=False)
print('Question:', prompt) # display the question
print('Answer:', answer) # display the answer

```
```bash
Question: Hello! Who are you?                                                                                                                                   
Answer: Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
Use the "print_predict" method to print the prediction response directly to the terminal screen without having to display the result manually.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
# the "print_predict" method has the same parameters with the same default values as the "predict" method
# note: the default value of the "prompt" parameter is an empty string
gptmodel.print_predict(prompt='Hello! Who are you?', max_tokens=500, temperature=0.5, top_k=0, top_p=1.0, stream=False) # prints the answer directly to the screen

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
Now check out what the display would look like with "stream" enabled.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
gptmodel.print_predict(prompt='Hello! Who are you?', stream=True)

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
When "end_tag" is set in training, you don't need to worry about the maximum number of tokens in the response.
In this case you can set a high threshold to get a more complete response.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
# since we defined a responses termination tag in our training, you can use a very high value in "max_tokens" to ensure a complete response and the response will still end at the correct point
gptmodel.print_predict(prompt='Hello! Who are you?', max_tokens=1000000, stream=True) # we set one million in "max_tokens" to ensure a complete response (this can only be done when "end_tag" is set in training)

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
Use the "precision" parameter to set a desired precision for the end of training, thus making your training faster and more dynamic.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# you can control the final precision of your training with a maximum value of cutoff, for this use the parameter named "precision"
# the "precision" parameter receives a floating number on a scale between 0 and 1 that represents the percentage of maximum desired precision
# when the training precision is greater than or equal to the precision of the "precision" parameter, the training will be finished with the last precision found
# the default value of the "precision" parameter is 0.9, that is, training will only be completed when an accuracy greater than or equal to 90% is found or when the internal tuning limit is exceeded
# setting "precision" to a value less than 1 will cause the training to complete faster, saving time and processing
# precisions above 0.8 may make your model overfitted and insensitive to adaptations, causing it to lose the ability to generalize to prediction inputs that are very different from the training inputs
# there may be some cases where lower precisions with values close to 0.5 are desirable
gptmodel.train('./dataset.json', precision=0.8) # training will be terminated when accuracy of 0.8 (80%) is reached
gptmodel.saveModel('./models/my_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.0it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.6it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000031 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000033 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000034 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000036 - current precision is 0.0000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000037 - current precision is 0.0093; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000038 - current precision is 0.0781; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000039 - current precision is 0.1248; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000040 - current precision is 0.1682; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000041 - current precision is 0.2140; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000042 - current precision is 0.2213; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000043 - current precision is 0.2966; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000044 - current precision is 0.3041; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000045 - current precision is 0.3509; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000046 - current precision is 0.3959; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000047 - current precision is 0.3960; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000048 - current precision is 0.4445; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000049 - current precision is 0.4453; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000050 - current precision is 0.4675; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000051 - current precision is 0.4886; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000052 - current precision is 0.5000; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000053 - current precision is 0.5250; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000054 - current precision is 0.5578; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000055 - current precision is 0.5301; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000056 - current precision is 0.5888; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000057 - current precision is 0.5942; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000058 - current precision is 0.5470; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000059 - current precision is 0.5966; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000060 - current precision is 0.5805; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000061 - current precision is 0.6220; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000062 - current precision is 0.6048; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000063 - current precision is 0.6171; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000064 - current precision is 0.6471; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000065 - current precision is 0.6563; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000066 - current precision is 0.6679; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000067 - current precision is 0.6581; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000068 - current precision is 0.6579; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000069 - current precision is 0.7061; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000070 - current precision is 0.6783; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000071 - current precision is 0.7146; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000072 - current precision is 0.7094; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000073 - current precision is 0.7142; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000074 - current precision is 0.7093; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000075 - current precision is 0.7379; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000076 - current precision is 0.7099; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000077 - current precision is 0.7082; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000078 - current precision is 0.7639; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000079 - current precision is 0.7310; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000080 - current precision is 0.7733; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000081 - current precision is 0.7780; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000082 - current precision is 0.7597; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000083 - current precision is 0.8077; aiming for precision >= 0.8000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 39.9kit/s

```
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
gptmodel.print_predict(prompt='Hello! Who are you?', stream=True)

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
With the "tokenizer" parameter you can choose between the Transformers' GPT tokenization pattern that processes pieces of words or Sapiens Technology's SAPI that processes each character individually to increase the level of abstraction in the learning process.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# the "tokenizer" parameter can receive a string with the value "gpt" or with the value "sapi"
# the default value of the tokenizer is "gpt" for a faster but less accurate training
# when setting the tokenizer to "sapi" the training will be a little slower because the knowledge abstraction will be done at the character level and not at the level of pieces of words
# "gpt" tends to be faster and less accurate, while "sapi" tends to be more accurate and less fast
# the choice of tokenizer will depend on your specific case and the needs of your project
gptmodel.train('./dataset.json', tokenizer='sapi', precision=0.9) # the sapi tokenizer should be avoided when training with very small datasets
gptmodel.saveModel('./models/my_model.gpt')
# the sapi tokenizer does not handle very small data sets well, and may return nonsensical responses and repeated characters
# to use the sapi tokenizer on modest datasets, we recommend keeping the precision value higher

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.49it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.49it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.48it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.49it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.48it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.48it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.48it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.48it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.48it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.48it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.47it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.46it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0020; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.46it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0563; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.46it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0682; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.46it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0738; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.46it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0922; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.46it/s]
Backpropagation epoch: 0000000029 - current precision is 0.1167; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.46it/s]
Backpropagation epoch: 0000000030 - current precision is 0.1898; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.46it/s]
Backpropagation epoch: 0000000031 - current precision is 0.2060; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.46it/s]
Backpropagation epoch: 0000000032 - current precision is 0.2194; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.46it/s]
Backpropagation epoch: 0000000033 - current precision is 0.2721; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.45it/s]
Backpropagation epoch: 0000000034 - current precision is 0.2864; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.45it/s]
Backpropagation epoch: 0000000035 - current precision is 0.3364; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.45it/s]
Backpropagation epoch: 0000000036 - current precision is 0.3983; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.45it/s]
Backpropagation epoch: 0000000037 - current precision is 0.3893; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.45it/s]
Backpropagation epoch: 0000000038 - current precision is 0.4013; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.45it/s]
Backpropagation epoch: 0000000039 - current precision is 0.4950; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.44it/s]
Backpropagation epoch: 0000000040 - current precision is 0.4841; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.44it/s]
Backpropagation epoch: 0000000041 - current precision is 0.5090; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.44it/s]
Backpropagation epoch: 0000000042 - current precision is 0.5684; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.44it/s]
Backpropagation epoch: 0000000043 - current precision is 0.6078; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.44it/s]
Backpropagation epoch: 0000000044 - current precision is 0.5708; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.43it/s]
Backpropagation epoch: 0000000045 - current precision is 0.6084; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.43it/s]
Backpropagation epoch: 0000000046 - current precision is 0.6426; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.42it/s]
Backpropagation epoch: 0000000047 - current precision is 0.6429; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.41it/s]
Backpropagation epoch: 0000000048 - current precision is 0.6766; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.41it/s]
Backpropagation epoch: 0000000049 - current precision is 0.6999; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.40it/s]
Backpropagation epoch: 0000000050 - current precision is 0.6803; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.40it/s]
Backpropagation epoch: 0000000051 - current precision is 0.7336; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.40it/s]
Backpropagation epoch: 0000000052 - current precision is 0.7592; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.40it/s]
Backpropagation epoch: 0000000053 - current precision is 0.7723; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.40it/s]
Backpropagation epoch: 0000000054 - current precision is 0.7912; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.39it/s]
Backpropagation epoch: 0000000055 - current precision is 0.8114; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.40it/s]
Backpropagation epoch: 0000000056 - current precision is 0.8120; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.40it/s]
Backpropagation epoch: 0000000057 - current precision is 0.8381; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.40it/s]
Backpropagation epoch: 0000000058 - current precision is 0.8391; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.41it/s]
Backpropagation epoch: 0000000059 - current precision is 0.8549; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.41it/s]
Backpropagation epoch: 0000000060 - current precision is 0.8865; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.41it/s]
Backpropagation epoch: 0000000061 - current precision is 0.8822; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.41it/s]
Backpropagation epoch: 0000000062 - current precision is 0.8736; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.41it/s]
Backpropagation epoch: 0000000063 - current precision is 0.8926; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.41it/s]
Backpropagation epoch: 0000000064 - current precision is 0.8907; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.42it/s]
Backpropagation epoch: 0000000065 - current precision is 0.8804; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.42it/s]
Backpropagation epoch: 0000000066 - current precision is 0.8969; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:01<00:00,  2.42it/s]
Backpropagation epoch: 0000000067 - current precision is 0.9141; aiming for precision >= 0.9000 in training: 100%|██████████| 3.00/3.00 [00:00<00:00,  131kit/s]

```
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
# prefer to set a maximum token limit in models trained with the sapi tokenizer to avoid excessive characters
gptmodel.print_predict(prompt='Hello!', max_tokens=50, stream=True)

```
```bash
 I am Sapiens Chat, an AI model created by Sapiens Technology.
```
The "context_window" training parameter is used to set the context window limit for your model. The default value for this parameter is 500 and when an integer greater than zero is provided, the model will use this value as the maximum input token limit for the prompt and the maximum token limit for the context.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# there is no maximum limit for the value of "context_window", but the higher this value is, the slower the training will be and also the slower the inference will be
gptmodel.train('./dataset.json', context_window=1024) # training with 1024 tokens context window
gptmodel.saveModel('./models/my_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  16.8it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.6it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.6it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.7it/s]
Backpropagation epoch: 0000000031 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000033 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000034 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000036 - current precision is 0.0695; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000037 - current precision is 0.1118; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000038 - current precision is 0.1862; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000039 - current precision is 0.2108; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000040 - current precision is 0.2589; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000041 - current precision is 0.3039; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000042 - current precision is 0.3406; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000043 - current precision is 0.3623; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000044 - current precision is 0.3996; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000045 - current precision is 0.4203; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000046 - current precision is 0.4698; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000047 - current precision is 0.4759; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000048 - current precision is 0.5060; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000049 - current precision is 0.5452; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000050 - current precision is 0.5595; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000051 - current precision is 0.5925; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000052 - current precision is 0.5668; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000053 - current precision is 0.5867; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000054 - current precision is 0.6379; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000055 - current precision is 0.6143; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000056 - current precision is 0.6241; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000057 - current precision is 0.6517; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000058 - current precision is 0.6505; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000059 - current precision is 0.6652; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000060 - current precision is 0.6861; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000061 - current precision is 0.6991; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000062 - current precision is 0.6807; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000063 - current precision is 0.5701; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000064 - current precision is 0.6770; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000065 - current precision is 0.5521; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000066 - current precision is 0.5899; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000067 - current precision is 0.6839; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000068 - current precision is 0.6577; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000069 - current precision is 0.6942; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000070 - current precision is 0.7019; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000071 - current precision is 0.6577; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000072 - current precision is 0.5770; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000073 - current precision is 0.6447; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000074 - current precision is 0.6032; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000075 - current precision is 0.6242; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000076 - current precision is 0.6835; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000077 - current precision is 0.6810; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000078 - current precision is 0.6799; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000079 - current precision is 0.6988; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000080 - current precision is 0.6916; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000081 - current precision is 0.7291; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000082 - current precision is 0.7117; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000083 - current precision is 0.7416; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000084 - current precision is 0.7332; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.2it/s]
Backpropagation epoch: 0000000085 - current precision is 0.7348; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000086 - current precision is 0.7525; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000087 - current precision is 0.7613; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000088 - current precision is 0.7727; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.1it/s]
Backpropagation epoch: 0000000089 - current precision is 0.7904; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000090 - current precision is 0.7956; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000091 - current precision is 0.8090; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000092 - current precision is 0.7834; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000093 - current precision is 0.8189; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000094 - current precision is 0.8363; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000095 - current precision is 0.7979; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000096 - current precision is 0.8009; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000097 - current precision is 0.8007; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000098 - current precision is 0.8289; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000099 - current precision is 0.8150; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000100 - current precision is 0.8526; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000101 - current precision is 0.8074; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000102 - current precision is 0.8230; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000103 - current precision is 0.8338; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000104 - current precision is 0.8291; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000105 - current precision is 0.7983; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000106 - current precision is 0.8265; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000107 - current precision is 0.8875; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000108 - current precision is 0.8448; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000109 - current precision is 0.8553; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000110 - current precision is 0.8440; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000111 - current precision is 0.7649; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000112 - current precision is 0.8506; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000113 - current precision is 0.8482; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000114 - current precision is 0.8128; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000115 - current precision is 0.8560; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000116 - current precision is 0.8976; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000117 - current precision is 0.8776; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000118 - current precision is 0.9055; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 33.3kit/s]

```
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
# since the model was trained with a context window of 1024 tokens, your prompt will be truncated considering only the final 1024 tokens when the input exceeds this limit
gptmodel.print_predict(prompt='Hello! Who are you?', stream=True) # input must be less than 1025 tokens

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
With the "validate" parameter you can use a float value greater than zero to enable a certain percentage amount (between 0 and 1) used to test/validate the training result and assign a value to the validation loss rate.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# if the "validate" parameter remains at its default value of 0.0, no validation will be performed at the end of training and both "val_loss" and "generalization_rate" will not change
# leaving validation disabled with 0.0 may make training complete faster, but it will make the saved model larger because all data will be included in the training
# use the validation when you want to know the real rate of "val_loss" and "generalization_rate"
# in the example below we are setting "validate" to 0.2, that is, 80% of the data will be used for training and 20% will be separated for testing
gptmodel.train('./dataset.json', validate=0.2)
gptmodel.saveModel('./models/my_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.2it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.4it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.4it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.4it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.2it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.3it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0382; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.4it/s]
Backpropagation epoch: 0000000008 - current precision is 0.2822; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.4it/s]
Backpropagation epoch: 0000000009 - current precision is 0.5816; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.4it/s]
Backpropagation epoch: 0000000010 - current precision is 0.5809; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.4it/s]
Backpropagation epoch: 0000000011 - current precision is 0.5665; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.4it/s]
Backpropagation epoch: 0000000012 - current precision is 0.8690; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.4it/s]
Backpropagation epoch: 0000000013 - current precision is 0.8137; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.4it/s]
Backpropagation epoch: 0000000014 - current precision is 0.7974; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  15.3it/s]
Backpropagation epoch: 0000000015 - current precision is 0.9392; aiming for precision >= 0.9000 in training: 100%|██████████| 4.00/4.00 [00:00<00:00,  266kit/s]

```
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
gptmodel.print_predict(prompt='What is the main language spoken in Spain?', stream=True)

```
```bash
The main language spoken in Spain is Spanish.
```
Check out an example below with all the training method parameters in their respective orders and their respective default values.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# the "dataset_path" and "string" parameters have an empty string as default value
gptmodel.train( # function for model training
    dataset_path='./dataset.json', # path to the training dataset in json or txt format
    string='', # string with optional complementary text for training
    precision=0.9, # approximate desired "precision/accuracy"
    tokenizer='gpt', # tokenizer for converting text into embeddings vectors
    context_window=500, # desired context window size
    end_tag=None, # subtext used as separator of input and output pairs in the file and/or training string
    validate=0.0, # percentage between 0 and 1 for the amount of training data that will be separated for validation/testing
    progress=True # enables or disables the visualization of training progress
) # returns a dictionary with the training metrics
gptmodel.saveModel('./models/my_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  17.5it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.9it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000031 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000033 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000034 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000036 - current precision is 0.0619; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000037 - current precision is 0.0699; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000038 - current precision is 0.1652; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000039 - current precision is 0.1800; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000040 - current precision is 0.2467; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000041 - current precision is 0.2854; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000042 - current precision is 0.3248; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000043 - current precision is 0.3534; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000044 - current precision is 0.3925; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000045 - current precision is 0.4266; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000046 - current precision is 0.4297; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000047 - current precision is 0.4681; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000048 - current precision is 0.4793; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000049 - current precision is 0.5098; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000050 - current precision is 0.5387; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000051 - current precision is 0.5422; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000052 - current precision is 0.5634; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000053 - current precision is 0.5879; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000054 - current precision is 0.5978; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000055 - current precision is 0.5911; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000056 - current precision is 0.5929; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000057 - current precision is 0.5848; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000058 - current precision is 0.6393; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000059 - current precision is 0.6296; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000060 - current precision is 0.6361; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000061 - current precision is 0.6511; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000062 - current precision is 0.6224; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000063 - current precision is 0.6770; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000064 - current precision is 0.6319; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000065 - current precision is 0.6494; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000066 - current precision is 0.6568; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000067 - current precision is 0.6737; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000068 - current precision is 0.6810; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000069 - current precision is 0.7025; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000070 - current precision is 0.7126; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000071 - current precision is 0.7209; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000072 - current precision is 0.6670; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000073 - current precision is 0.7268; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000074 - current precision is 0.6629; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000075 - current precision is 0.7065; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000076 - current precision is 0.7268; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000077 - current precision is 0.7393; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000078 - current precision is 0.7552; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000079 - current precision is 0.7523; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000080 - current precision is 0.7566; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000081 - current precision is 0.7506; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000082 - current precision is 0.7569; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000083 - current precision is 0.7987; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000084 - current precision is 0.7965; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000085 - current precision is 0.7302; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000086 - current precision is 0.7826; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000087 - current precision is 0.6665; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000088 - current precision is 0.7207; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000089 - current precision is 0.6800; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000090 - current precision is 0.7732; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000091 - current precision is 0.7935; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000092 - current precision is 0.7905; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000093 - current precision is 0.7799; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000094 - current precision is 0.8096; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000095 - current precision is 0.8281; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000096 - current precision is 0.7801; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000097 - current precision is 0.7393; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000098 - current precision is 0.8189; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000099 - current precision is 0.7831; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000100 - current precision is 0.8198; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000101 - current precision is 0.8352; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000102 - current precision is 0.7970; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000103 - current precision is 0.7602; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000104 - current precision is 0.7992; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000105 - current precision is 0.8237; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  18.7it/s]
Backpropagation epoch: 0000000106 - current precision is 0.8248; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000107 - current precision is 0.8515; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.7it/s]
Backpropagation epoch: 0000000108 - current precision is 0.8771; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000109 - current precision is 0.8604; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000110 - current precision is 0.8675; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000111 - current precision is 0.8815; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000112 - current precision is 0.8445; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000113 - current precision is 0.9012; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 34.7kit/s]

```
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt') # always load the model before running the "predict" or "print_predict" function
gptmodel.print_predict( # displays the response to the input prompt
    prompt='Hello! Who are you?', # input for which you want to obtain an output
    max_tokens=500, # approximate maximum number of tokens in the response
    temperature=0.5, # inference temperature for the level of variability in responses
    top_k=0, # size of the set of likely tokens for random selection
    top_p=1.0, # minimum probability for a token to be included in the random selection
    stream=True # enables or disables real-time display on a token-by-token basis
)

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
The model training hyperparameters can be configured in the class constructor, or if you prefer, you can keep the default values.
```python
from gptmodel import GPTModel
# setting hyperparameter values in the class constructor causes the default values to be overwritten
# if one or more hyperparameters are not defined in the constructor, training will run with the default value of the hyperparameter that has not been changed
# you can continue with the default values, or you can customize them according to your needs
# below is an example with all hyperparameters being configured manually
gptmodel = GPTModel( # object class constructor
    embedding_dim=256, # amount of elements per token vector, each token will be converted into a vector with that amount of numbers
    block_size=64, # maximum number of tokens for the context window size
    batch_size=8, # maximum number of input-output pairs read at once during training
    number_heads=4, # number of attention mechanisms paying attention to different parts of the text at the same time
    number_layers=4, # number of hidden layers in the network, the higher this number, the greater the pattern abstraction
    dropout=0.2, # percentage (between 0 and 1) of neurons turned off at each step to prevent the model from memorizing the answers and actually learn instead of just memorizing
    learning_rate=0.0001, # learning speed, if too high the model learns little and quickly, if too low it learns a lot but slowly (preferably between 0 and 1)
    eval_interval=200, # number of steps in each backpropagation until the next validation
    epochs=500 # number of times all data is read for learning, if too high the model runs the risk of starting to memorize instead of learning
)
gptmodel.train('./dataset.json')
gptmodel.saveModel('./models/my_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  24.2it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  27.7it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.6it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.3it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.3it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.4it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.5it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000031 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000033 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000034 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.6it/s]
Backpropagation epoch: 0000000036 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000037 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000038 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000039 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000040 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000041 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000042 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000043 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000044 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000045 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000046 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000047 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000048 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000049 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000050 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000051 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000052 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000053 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000054 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000055 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000056 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.6it/s]
Backpropagation epoch: 0000000057 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000058 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000059 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.2it/s]
Backpropagation epoch: 0000000060 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000061 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.2it/s]
Backpropagation epoch: 0000000062 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000063 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000064 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000065 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000066 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000067 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000068 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000069 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000070 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000071 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000072 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000073 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.2it/s]
Backpropagation epoch: 0000000074 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000075 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000076 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000077 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000078 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000079 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000080 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000081 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000082 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000083 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000084 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000085 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000086 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000087 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000088 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000089 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000090 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000091 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000092 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000093 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000094 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000095 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000096 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000097 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000098 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000099 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000100 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000101 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000102 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000103 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000104 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000105 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000106 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000107 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000108 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000109 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000110 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000111 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000112 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000113 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000114 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000115 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000116 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000117 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000118 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000119 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000120 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000121 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.8it/s]
Backpropagation epoch: 0000000122 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000123 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000124 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000125 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.1it/s]
Backpropagation epoch: 0000000126 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  27.7it/s]
Backpropagation epoch: 0000000127 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000128 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000129 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000130 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.4it/s]
Backpropagation epoch: 0000000131 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000132 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000133 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000134 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000135 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000136 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.4it/s]
Backpropagation epoch: 0000000137 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000138 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000139 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000140 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000141 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000142 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000143 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000144 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000145 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000146 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000147 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000148 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000149 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000150 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000151 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000152 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000153 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000154 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000155 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000156 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000157 - current precision is 0.0158; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000158 - current precision is 0.0347; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000159 - current precision is 0.0784; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000160 - current precision is 0.0902; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000161 - current precision is 0.0855; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000162 - current precision is 0.0990; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000163 - current precision is 0.1256; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000164 - current precision is 0.1448; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000165 - current precision is 0.1445; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000166 - current precision is 0.1661; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000167 - current precision is 0.1691; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.5it/s]
Backpropagation epoch: 0000000168 - current precision is 0.2093; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.5it/s]
Backpropagation epoch: 0000000169 - current precision is 0.2058; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000170 - current precision is 0.2156; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000171 - current precision is 0.2240; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000172 - current precision is 0.2293; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000173 - current precision is 0.2225; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000174 - current precision is 0.2384; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000175 - current precision is 0.2610; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000176 - current precision is 0.2500; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000177 - current precision is 0.2791; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000178 - current precision is 0.2639; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000179 - current precision is 0.2931; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000180 - current precision is 0.3105; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000181 - current precision is 0.2977; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000182 - current precision is 0.3277; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000183 - current precision is 0.3254; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000184 - current precision is 0.3317; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000185 - current precision is 0.3312; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  26.1it/s]
Backpropagation epoch: 0000000186 - current precision is 0.3606; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.0it/s]
Backpropagation epoch: 0000000187 - current precision is 0.3695; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.4it/s]
Backpropagation epoch: 0000000188 - current precision is 0.3759; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000189 - current precision is 0.3697; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000190 - current precision is 0.3609; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000191 - current precision is 0.3716; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000192 - current precision is 0.3960; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000193 - current precision is 0.4034; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000194 - current precision is 0.4264; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000195 - current precision is 0.3954; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000196 - current precision is 0.4317; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000197 - current precision is 0.4031; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000198 - current precision is 0.4509; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.6it/s]
Backpropagation epoch: 0000000199 - current precision is 0.4183; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000200 - current precision is 0.4467; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000201 - current precision is 0.4310; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.4it/s]
Backpropagation epoch: 0000000202 - current precision is 0.4332; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000203 - current precision is 0.4701; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000204 - current precision is 0.4864; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000205 - current precision is 0.4593; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000206 - current precision is 0.4785; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000207 - current precision is 0.4453; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000208 - current precision is 0.4710; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000209 - current precision is 0.4961; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000210 - current precision is 0.5062; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000211 - current precision is 0.5097; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000212 - current precision is 0.5118; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000213 - current precision is 0.4923; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000214 - current precision is 0.5078; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000215 - current precision is 0.4784; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000216 - current precision is 0.4817; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000217 - current precision is 0.5413; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000218 - current precision is 0.4971; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000219 - current precision is 0.5247; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000220 - current precision is 0.5113; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000221 - current precision is 0.5462; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000222 - current precision is 0.5283; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000223 - current precision is 0.5579; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000224 - current precision is 0.5297; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000225 - current precision is 0.5132; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000226 - current precision is 0.5660; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000227 - current precision is 0.5314; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000228 - current precision is 0.5371; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000229 - current precision is 0.5049; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000230 - current precision is 0.5456; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000231 - current precision is 0.5525; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000232 - current precision is 0.5600; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000233 - current precision is 0.5441; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000234 - current precision is 0.5844; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000235 - current precision is 0.5753; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000236 - current precision is 0.5540; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000237 - current precision is 0.5919; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000238 - current precision is 0.5535; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000239 - current precision is 0.6073; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000240 - current precision is 0.5684; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000241 - current precision is 0.6056; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000242 - current precision is 0.6078; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000243 - current precision is 0.5731; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.4it/s]
Backpropagation epoch: 0000000244 - current precision is 0.5591; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000245 - current precision is 0.5842; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000246 - current precision is 0.6088; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000247 - current precision is 0.5921; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000248 - current precision is 0.5752; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000249 - current precision is 0.5945; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000250 - current precision is 0.5649; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000251 - current precision is 0.6028; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000252 - current precision is 0.6187; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000253 - current precision is 0.5966; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000254 - current precision is 0.5531; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000255 - current precision is 0.6032; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000256 - current precision is 0.5966; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000257 - current precision is 0.6462; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000258 - current precision is 0.6074; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000259 - current precision is 0.5852; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000260 - current precision is 0.6311; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000261 - current precision is 0.6267; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000262 - current precision is 0.6286; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000263 - current precision is 0.6477; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000264 - current precision is 0.6305; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000265 - current precision is 0.6248; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000266 - current precision is 0.6438; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000267 - current precision is 0.6202; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000268 - current precision is 0.6618; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000269 - current precision is 0.6469; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000270 - current precision is 0.6645; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000271 - current precision is 0.6610; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000272 - current precision is 0.6371; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000273 - current precision is 0.6558; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000274 - current precision is 0.6669; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000275 - current precision is 0.6566; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000276 - current precision is 0.6597; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000277 - current precision is 0.6714; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000278 - current precision is 0.6960; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000279 - current precision is 0.6689; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000280 - current precision is 0.6742; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000281 - current precision is 0.7023; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000282 - current precision is 0.6831; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000283 - current precision is 0.6942; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000284 - current precision is 0.6702; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000285 - current precision is 0.6782; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000286 - current precision is 0.6772; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000287 - current precision is 0.6781; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000288 - current precision is 0.6969; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000289 - current precision is 0.6919; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000290 - current precision is 0.6953; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000291 - current precision is 0.6970; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000292 - current precision is 0.6999; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000293 - current precision is 0.7160; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000294 - current precision is 0.6914; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000295 - current precision is 0.7103; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000296 - current precision is 0.7205; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000297 - current precision is 0.7270; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000298 - current precision is 0.7122; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000299 - current precision is 0.7192; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000300 - current precision is 0.7249; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000301 - current precision is 0.7147; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000302 - current precision is 0.6554; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000303 - current precision is 0.7123; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000304 - current precision is 0.7020; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.6it/s]
Backpropagation epoch: 0000000305 - current precision is 0.7361; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000306 - current precision is 0.6927; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000307 - current precision is 0.7300; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000308 - current precision is 0.7398; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000309 - current precision is 0.6737; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000310 - current precision is 0.7137; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000311 - current precision is 0.7589; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000312 - current precision is 0.7297; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000313 - current precision is 0.7136; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000314 - current precision is 0.6796; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000315 - current precision is 0.7611; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000316 - current precision is 0.7285; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000317 - current precision is 0.7612; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000318 - current precision is 0.6761; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000319 - current precision is 0.6778; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000320 - current precision is 0.6853; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000321 - current precision is 0.7048; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000322 - current precision is 0.7155; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000323 - current precision is 0.7158; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000324 - current precision is 0.7188; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000325 - current precision is 0.7467; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000326 - current precision is 0.7359; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000327 - current precision is 0.7423; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000328 - current precision is 0.7455; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000329 - current precision is 0.7521; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000330 - current precision is 0.7366; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000331 - current precision is 0.7212; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000332 - current precision is 0.7468; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.7it/s]
Backpropagation epoch: 0000000333 - current precision is 0.7717; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000334 - current precision is 0.7493; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.6it/s]
Backpropagation epoch: 0000000335 - current precision is 0.7867; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000336 - current precision is 0.7646; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000337 - current precision is 0.7191; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000338 - current precision is 0.7415; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000339 - current precision is 0.7516; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000340 - current precision is 0.7856; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000341 - current precision is 0.7642; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000342 - current precision is 0.7522; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000343 - current precision is 0.7621; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000344 - current precision is 0.7391; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000345 - current precision is 0.7651; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000346 - current precision is 0.7466; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000347 - current precision is 0.7968; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000348 - current precision is 0.7257; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000349 - current precision is 0.7923; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000350 - current precision is 0.7570; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000351 - current precision is 0.7785; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000352 - current precision is 0.7725; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000353 - current precision is 0.8032; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000354 - current precision is 0.7493; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000355 - current precision is 0.7941; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000356 - current precision is 0.7920; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000357 - current precision is 0.7657; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000358 - current precision is 0.8029; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000359 - current precision is 0.7902; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000360 - current precision is 0.8098; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000361 - current precision is 0.7752; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000362 - current precision is 0.7937; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000363 - current precision is 0.7869; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000364 - current precision is 0.8007; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000365 - current precision is 0.7865; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000366 - current precision is 0.8068; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000367 - current precision is 0.7708; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000368 - current precision is 0.7864; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000369 - current precision is 0.8182; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000370 - current precision is 0.7929; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000371 - current precision is 0.8022; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  26.3it/s]
Backpropagation epoch: 0000000372 - current precision is 0.8126; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.0it/s]
Backpropagation epoch: 0000000373 - current precision is 0.8174; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000374 - current precision is 0.8126; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.8it/s]
Backpropagation epoch: 0000000375 - current precision is 0.8193; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.2it/s]
Backpropagation epoch: 0000000376 - current precision is 0.8325; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000377 - current precision is 0.8033; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.4it/s]
Backpropagation epoch: 0000000378 - current precision is 0.8068; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000379 - current precision is 0.8417; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000380 - current precision is 0.8224; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000381 - current precision is 0.7839; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000382 - current precision is 0.8221; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000383 - current precision is 0.8138; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000384 - current precision is 0.8144; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000385 - current precision is 0.8144; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000386 - current precision is 0.8312; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.8it/s]
Backpropagation epoch: 0000000387 - current precision is 0.8414; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000388 - current precision is 0.7971; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000389 - current precision is 0.8131; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000390 - current precision is 0.8316; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000391 - current precision is 0.8326; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000392 - current precision is 0.8175; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.8it/s]
Backpropagation epoch: 0000000393 - current precision is 0.8472; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000394 - current precision is 0.8517; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000395 - current precision is 0.8391; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000396 - current precision is 0.8041; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000397 - current precision is 0.8144; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.4it/s]
Backpropagation epoch: 0000000398 - current precision is 0.8251; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.5it/s]
Backpropagation epoch: 0000000399 - current precision is 0.8748; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000400 - current precision is 0.8418; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000401 - current precision is 0.8177; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000402 - current precision is 0.8217; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.8it/s]
Backpropagation epoch: 0000000403 - current precision is 0.8067; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000404 - current precision is 0.7994; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000405 - current precision is 0.7714; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000406 - current precision is 0.8154; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000407 - current precision is 0.8181; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000408 - current precision is 0.8255; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000409 - current precision is 0.7775; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000410 - current precision is 0.8304; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000411 - current precision is 0.8367; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000412 - current precision is 0.8266; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000413 - current precision is 0.7975; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000414 - current precision is 0.8049; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000415 - current precision is 0.8691; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000416 - current precision is 0.7932; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000417 - current precision is 0.8318; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000418 - current precision is 0.7987; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000419 - current precision is 0.8508; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000420 - current precision is 0.8441; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000421 - current precision is 0.8500; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000422 - current precision is 0.8440; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000423 - current precision is 0.8597; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000424 - current precision is 0.8714; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000425 - current precision is 0.8030; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000426 - current precision is 0.8262; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000427 - current precision is 0.8268; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.5it/s]
Backpropagation epoch: 0000000428 - current precision is 0.8084; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000429 - current precision is 0.7941; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000430 - current precision is 0.8492; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000431 - current precision is 0.7829; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.5it/s]
Backpropagation epoch: 0000000432 - current precision is 0.8240; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000433 - current precision is 0.8659; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000434 - current precision is 0.8690; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000435 - current precision is 0.8012; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000436 - current precision is 0.8504; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000437 - current precision is 0.8750; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000438 - current precision is 0.8625; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000439 - current precision is 0.8336; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000440 - current precision is 0.8625; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000441 - current precision is 0.8626; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.8it/s]
Backpropagation epoch: 0000000442 - current precision is 0.8437; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000443 - current precision is 0.8584; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000444 - current precision is 0.8416; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000445 - current precision is 0.8893; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000446 - current precision is 0.8545; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.1it/s]
Backpropagation epoch: 0000000447 - current precision is 0.8583; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000448 - current precision is 0.8311; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000449 - current precision is 0.8510; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000450 - current precision is 0.8150; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000451 - current precision is 0.7665; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.4it/s]
Backpropagation epoch: 0000000452 - current precision is 0.8056; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000453 - current precision is 0.8364; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000454 - current precision is 0.8561; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000455 - current precision is 0.8642; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000456 - current precision is 0.8654; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.5it/s]
Backpropagation epoch: 0000000457 - current precision is 0.8337; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000458 - current precision is 0.8638; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  27.8it/s]
Backpropagation epoch: 0000000459 - current precision is 0.8742; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000460 - current precision is 0.8356; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.6it/s]
Backpropagation epoch: 0000000461 - current precision is 0.8664; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000462 - current precision is 0.8784; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000463 - current precision is 0.8648; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.8it/s]
Backpropagation epoch: 0000000464 - current precision is 0.8692; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000465 - current precision is 0.8773; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000466 - current precision is 0.8708; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000467 - current precision is 0.8792; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000468 - current precision is 0.8531; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000469 - current precision is 0.8755; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000470 - current precision is 0.8826; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000471 - current precision is 0.8476; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000472 - current precision is 0.8878; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.6it/s]
Backpropagation epoch: 0000000473 - current precision is 0.8769; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.6it/s]
Backpropagation epoch: 0000000474 - current precision is 0.8663; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000475 - current precision is 0.8808; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.9it/s]
Backpropagation epoch: 0000000476 - current precision is 0.8680; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000477 - current precision is 0.8908; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000478 - current precision is 0.8876; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000479 - current precision is 0.8806; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.6it/s]
Backpropagation epoch: 0000000480 - current precision is 0.8704; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000481 - current precision is 0.8619; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000482 - current precision is 0.8658; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.7it/s]
Backpropagation epoch: 0000000483 - current precision is 0.8320; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.2it/s]
Backpropagation epoch: 0000000484 - current precision is 0.8760; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000485 - current precision is 0.8538; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  28.8it/s]
Backpropagation epoch: 0000000486 - current precision is 0.8709; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000487 - current precision is 0.8500; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000488 - current precision is 0.7781; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000489 - current precision is 0.8162; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.2it/s]
Backpropagation epoch: 0000000490 - current precision is 0.8707; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000491 - current precision is 0.8876; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000492 - current precision is 0.8951; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.1it/s]
Backpropagation epoch: 0000000493 - current precision is 0.8901; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000494 - current precision is 0.8804; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000495 - current precision is 0.9050; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 11.0kit/s]

```
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
gptmodel.print_predict(prompt='Who discovered Brazil?', stream=True)

```
```bash
Brazil was discovered by Pedro Alvares Cabral in 1500.
```
The value of the "dropout" parameter can also be changed through the class's public variable right after the constructor is instantiated.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.dropout = 0.2
gptmodel.train('./dataset.json')
gptmodel.saveModel('./models/my_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  16.8it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000031 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000033 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000034 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000036 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000037 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000038 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000039 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000040 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000041 - current precision is 0.0489; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000042 - current precision is 0.1061; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000043 - current precision is 0.1590; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000044 - current precision is 0.1926; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000045 - current precision is 0.2201; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000046 - current precision is 0.2621; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000047 - current precision is 0.2835; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000048 - current precision is 0.3039; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000049 - current precision is 0.3350; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000050 - current precision is 0.3733; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000051 - current precision is 0.4052; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000052 - current precision is 0.3926; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000053 - current precision is 0.4408; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000054 - current precision is 0.4410; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000055 - current precision is 0.4678; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000056 - current precision is 0.4510; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000057 - current precision is 0.4751; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000058 - current precision is 0.4919; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000059 - current precision is 0.4758; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000060 - current precision is 0.4885; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000061 - current precision is 0.5334; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000062 - current precision is 0.5611; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000063 - current precision is 0.5477; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000064 - current precision is 0.5760; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000065 - current precision is 0.5275; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000066 - current precision is 0.5662; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000067 - current precision is 0.5816; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000068 - current precision is 0.5606; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000069 - current precision is 0.5875; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000070 - current precision is 0.5721; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000071 - current precision is 0.5828; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000072 - current precision is 0.5676; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000073 - current precision is 0.6027; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000074 - current precision is 0.6070; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000075 - current precision is 0.6028; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000076 - current precision is 0.6121; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000077 - current precision is 0.6425; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000078 - current precision is 0.6480; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000079 - current precision is 0.6482; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000080 - current precision is 0.6193; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000081 - current precision is 0.6666; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000082 - current precision is 0.6605; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000083 - current precision is 0.6528; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000084 - current precision is 0.6670; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000085 - current precision is 0.6476; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000086 - current precision is 0.6750; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000087 - current precision is 0.6337; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000088 - current precision is 0.6049; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000089 - current precision is 0.6684; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000090 - current precision is 0.6349; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000091 - current precision is 0.5816; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000092 - current precision is 0.6709; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000093 - current precision is 0.6712; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000094 - current precision is 0.6933; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000095 - current precision is 0.6592; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000096 - current precision is 0.6873; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000097 - current precision is 0.7304; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000098 - current precision is 0.6952; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000099 - current precision is 0.7268; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000100 - current precision is 0.7413; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000101 - current precision is 0.7184; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000102 - current precision is 0.7383; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000103 - current precision is 0.7103; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000104 - current precision is 0.7043; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000105 - current precision is 0.7426; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000106 - current precision is 0.7479; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000107 - current precision is 0.7289; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000108 - current precision is 0.7519; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000109 - current precision is 0.7516; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000110 - current precision is 0.7223; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000111 - current precision is 0.7776; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000112 - current precision is 0.7847; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000113 - current precision is 0.7625; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000114 - current precision is 0.7837; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000115 - current precision is 0.7400; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000116 - current precision is 0.7633; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000117 - current precision is 0.7698; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000118 - current precision is 0.7338; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000119 - current precision is 0.7748; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000120 - current precision is 0.7527; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000121 - current precision is 0.8169; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000122 - current precision is 0.7868; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000123 - current precision is 0.7807; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000124 - current precision is 0.8004; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000125 - current precision is 0.8272; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000126 - current precision is 0.8311; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000127 - current precision is 0.7286; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000128 - current precision is 0.7471; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000129 - current precision is 0.7713; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.2it/s]
Backpropagation epoch: 0000000130 - current precision is 0.8022; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.4it/s]
Backpropagation epoch: 0000000131 - current precision is 0.7948; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000132 - current precision is 0.7771; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000133 - current precision is 0.8291; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000134 - current precision is 0.8385; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000135 - current precision is 0.8268; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000136 - current precision is 0.8149; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000137 - current precision is 0.8276; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000138 - current precision is 0.8140; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000139 - current precision is 0.8452; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000140 - current precision is 0.8324; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000141 - current precision is 0.8373; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000142 - current precision is 0.8901; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000143 - current precision is 0.8524; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000144 - current precision is 0.8756; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000145 - current precision is 0.8647; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000146 - current precision is 0.8859; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000147 - current precision is 0.8558; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000148 - current precision is 0.8943; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000149 - current precision is 0.7466; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000150 - current precision is 0.8602; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000151 - current precision is 0.8734; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000152 - current precision is 0.8377; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000153 - current precision is 0.8292; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000154 - current precision is 0.8579; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000155 - current precision is 0.8754; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000156 - current precision is 0.9018; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 23.7kit/s]

```
It is also possible to view the total number of model parameters through the "parameters_number" variable. This variable with the number of parameters can be accessed through the class object right after training or loading the model.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
parameters_number = gptmodel.parameters_number # get the total number of parameters of the trained model
print(f'The loaded model has {parameters_number} parameters in total.')
gptmodel.print_predict(prompt='Hello! Who are you?', stream=True)

```
```bash
The loaded model has 55273681 parameters in total.                                                                                                              
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
The "addFit" method when called before the "train" method, as well as the "string" parameter, can also be used to complement the training dataset.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# the "addFit" method is being used to complement the training data
# note: when called before training, it is not fine-tuning, as it will be part of the original data
gptmodel.addFit('What does the acronym GPT mean in language models?', 'GPT in language models stands for Generative Pre-trained Transformer.')
gptmodel.addFit('What is the capital of Spain?', 'The capital of Spain is Madrid.')
gptmodel.train('./dataset.json')
gptmodel.saveModel('./models/my_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  17.1it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.0it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.6it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000031 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000033 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000034 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.8it/s]
Backpropagation epoch: 0000000036 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000037 - current precision is 0.0331; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000038 - current precision is 0.0671; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000039 - current precision is 0.1481; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000040 - current precision is 0.1754; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000041 - current precision is 0.2187; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000042 - current precision is 0.2508; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000043 - current precision is 0.3006; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000044 - current precision is 0.3242; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000045 - current precision is 0.3415; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000046 - current precision is 0.3700; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000047 - current precision is 0.4120; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000048 - current precision is 0.4220; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000049 - current precision is 0.4585; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000050 - current precision is 0.4746; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000051 - current precision is 0.4482; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000052 - current precision is 0.4835; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000053 - current precision is 0.4743; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000054 - current precision is 0.4996; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000055 - current precision is 0.5231; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000056 - current precision is 0.5290; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000057 - current precision is 0.5580; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000058 - current precision is 0.5498; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000059 - current precision is 0.5767; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000060 - current precision is 0.6020; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000061 - current precision is 0.5956; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000062 - current precision is 0.5949; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000063 - current precision is 0.5759; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000064 - current precision is 0.6394; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000065 - current precision is 0.5525; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000066 - current precision is 0.5544; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000067 - current precision is 0.5644; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000068 - current precision is 0.5799; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000069 - current precision is 0.6374; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000070 - current precision is 0.6267; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000071 - current precision is 0.6582; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000072 - current precision is 0.6350; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000073 - current precision is 0.6286; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000074 - current precision is 0.6693; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000075 - current precision is 0.6534; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000076 - current precision is 0.6942; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000077 - current precision is 0.6825; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000078 - current precision is 0.6629; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000079 - current precision is 0.6946; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000080 - current precision is 0.7245; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000081 - current precision is 0.7085; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000082 - current precision is 0.7254; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000083 - current precision is 0.7090; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000084 - current precision is 0.7708; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000085 - current precision is 0.6582; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000086 - current precision is 0.6839; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000087 - current precision is 0.6593; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000088 - current precision is 0.7111; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000089 - current precision is 0.7427; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000090 - current precision is 0.7358; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000091 - current precision is 0.7616; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000092 - current precision is 0.7707; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000093 - current precision is 0.7486; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000094 - current precision is 0.8048; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000095 - current precision is 0.7489; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000096 - current precision is 0.7833; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000097 - current precision is 0.7810; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000098 - current precision is 0.7599; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000099 - current precision is 0.8202; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000100 - current precision is 0.8122; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000101 - current precision is 0.7881; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000102 - current precision is 0.8325; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000103 - current precision is 0.8446; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000104 - current precision is 0.8318; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000105 - current precision is 0.8666; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000106 - current precision is 0.8228; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000107 - current precision is 0.8507; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000108 - current precision is 0.8226; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000109 - current precision is 0.8747; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000110 - current precision is 0.8604; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000111 - current precision is 0.8701; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000112 - current precision is 0.8865; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000113 - current precision is 0.8687; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000114 - current precision is 0.9024; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 33.3kit/s]

```
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
gptmodel.print_predict('Hello! Who are you?', stream=True)

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.
```
The "addFit" method when invoked after a model training or loading, can be used to add fits to the trained or loaded model.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.train('./dataset.json')
# you can fine-tune a model right after training by calling the "addFit" method which will readjust the weights based on the new input-output pairs added
for _ in range(20): # we recommend placing your "addFit" method calls inside a loop with at least 20 repetitions, as for most cases, a single fine-tuning call is not enough
	gptmodel.addFit('What does the acronym GPT mean in language models?', 'GPT in language models stands for Generative Pre-trained Transformer.')
	gptmodel.addFit('What is the capital of Spain?', 'The capital of Spain is Madrid.')
# save the model with the training data adjusted with the two input and output pairs added above
# you can add as many pairs as you want to your fit
gptmodel.saveModel('./models/my_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  17.8it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000031 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000033 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000034 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000036 - current precision is 0.0496; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000037 - current precision is 0.1141; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000038 - current precision is 0.1526; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000039 - current precision is 0.1880; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000040 - current precision is 0.2364; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000041 - current precision is 0.2981; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000042 - current precision is 0.3255; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000043 - current precision is 0.3684; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000044 - current precision is 0.3920; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000045 - current precision is 0.4177; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000046 - current precision is 0.4408; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000047 - current precision is 0.4818; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000048 - current precision is 0.5051; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000049 - current precision is 0.5428; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000050 - current precision is 0.5460; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000051 - current precision is 0.5562; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000052 - current precision is 0.5417; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000053 - current precision is 0.5827; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000054 - current precision is 0.6196; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000055 - current precision is 0.6103; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000056 - current precision is 0.6383; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000057 - current precision is 0.6536; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000058 - current precision is 0.6215; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000059 - current precision is 0.6485; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000060 - current precision is 0.6066; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000061 - current precision is 0.6566; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000062 - current precision is 0.5861; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000063 - current precision is 0.6066; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000064 - current precision is 0.6589; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000065 - current precision is 0.6209; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000066 - current precision is 0.6590; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000067 - current precision is 0.6444; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000068 - current precision is 0.6916; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000069 - current precision is 0.6596; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000070 - current precision is 0.6594; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000071 - current precision is 0.6997; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.1it/s]
Backpropagation epoch: 0000000072 - current precision is 0.6990; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000073 - current precision is 0.6839; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000074 - current precision is 0.6855; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000075 - current precision is 0.6975; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000076 - current precision is 0.7318; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000077 - current precision is 0.7195; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000078 - current precision is 0.7537; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000079 - current precision is 0.7350; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000080 - current precision is 0.7772; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000081 - current precision is 0.7343; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000082 - current precision is 0.7556; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000083 - current precision is 0.7750; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000084 - current precision is 0.7572; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000085 - current precision is 0.8013; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000086 - current precision is 0.8060; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000087 - current precision is 0.8497; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000088 - current precision is 0.8181; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000089 - current precision is 0.8018; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000090 - current precision is 0.8065; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000091 - current precision is 0.8317; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000092 - current precision is 0.8543; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000093 - current precision is 0.8428; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000094 - current precision is 0.8267; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000095 - current precision is 0.8236; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000096 - current precision is 0.8089; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000097 - current precision is 0.7943; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000098 - current precision is 0.7666; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000099 - current precision is 0.8567; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.1it/s]
Backpropagation epoch: 0000000100 - current precision is 0.8544; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000101 - current precision is 0.8438; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000102 - current precision is 0.8426; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000103 - current precision is 0.8300; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000104 - current precision is 0.8531; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000105 - current precision is 0.8918; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000106 - current precision is 0.8503; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000107 - current precision is 0.8605; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000108 - current precision is 0.8864; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000109 - current precision is 0.8599; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000110 - current precision is 0.8763; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000111 - current precision is 0.9064; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 33.3kit/s]

```
If the inference answers are not satisfactory, repeat the fine-tuning with larger numbers of repetitions in the loop.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')

gptmodel.print_predict(prompt='Hello! Who are you?', stream=True)
gptmodel.print_predict(prompt='What does the acronym GPT mean in language models?', stream=True)
gptmodel.print_predict(prompt='What is the capital of Spain?', stream=True)

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.                                                                                            
GPT in language models stands for Generative Pre-trained Transformer.
The capital of Spain is Madrid.
```
Fine-tuning with the "addFit" method can also be done on a pre-trained model after loading.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.train('./dataset.json')
save_model = gptmodel.saveModel('./models/my_model.gpt') # returns True if the model is saved successfully, or False otherwise
if save_model: print(f'Model saved with {gptmodel.parameters_number} parameters.') # displays a save confirmation message
else: print('ERROR saving the model!!') # if an error occurs while saving, an error message is displayed.

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  17.7it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000019 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000020 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000021 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000022 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000024 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.2it/s]
Backpropagation epoch: 0000000026 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000027 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000029 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000030 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000031 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000033 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000034 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0000; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000036 - current precision is 0.0820; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000037 - current precision is 0.1153; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000038 - current precision is 0.1759; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000039 - current precision is 0.2410; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000040 - current precision is 0.2404; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000041 - current precision is 0.3004; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000042 - current precision is 0.2796; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000043 - current precision is 0.3685; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000044 - current precision is 0.3710; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000045 - current precision is 0.3863; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  19.9it/s]
Backpropagation epoch: 0000000046 - current precision is 0.4502; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000047 - current precision is 0.4406; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000048 - current precision is 0.4916; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000049 - current precision is 0.5099; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000050 - current precision is 0.5006; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000051 - current precision is 0.5409; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000052 - current precision is 0.5522; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000053 - current precision is 0.5356; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000054 - current precision is 0.5941; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000055 - current precision is 0.6007; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000056 - current precision is 0.5976; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000057 - current precision is 0.6166; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000058 - current precision is 0.6365; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000059 - current precision is 0.6219; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000060 - current precision is 0.5876; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000061 - current precision is 0.6757; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000062 - current precision is 0.5820; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000063 - current precision is 0.5937; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000064 - current precision is 0.6666; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000065 - current precision is 0.6366; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000066 - current precision is 0.6230; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000067 - current precision is 0.6613; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000068 - current precision is 0.6887; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000069 - current precision is 0.6756; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000070 - current precision is 0.6770; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000071 - current precision is 0.6865; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000072 - current precision is 0.6982; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000073 - current precision is 0.6957; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000074 - current precision is 0.6952; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.0it/s]
Backpropagation epoch: 0000000075 - current precision is 0.6744; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.3it/s]
Backpropagation epoch: 0000000076 - current precision is 0.7334; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000077 - current precision is 0.7314; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000078 - current precision is 0.7165; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000079 - current precision is 0.7448; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000080 - current precision is 0.7203; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000081 - current precision is 0.7312; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000082 - current precision is 0.7693; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000083 - current precision is 0.7104; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000084 - current precision is 0.7750; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000085 - current precision is 0.7560; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000086 - current precision is 0.7849; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000087 - current precision is 0.7760; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000088 - current precision is 0.7186; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000089 - current precision is 0.7920; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000090 - current precision is 0.7916; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000091 - current precision is 0.8103; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000092 - current precision is 0.7709; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000093 - current precision is 0.7691; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000094 - current precision is 0.8194; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.2it/s]
Backpropagation epoch: 0000000095 - current precision is 0.7331; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000096 - current precision is 0.7597; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000097 - current precision is 0.8037; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000098 - current precision is 0.7542; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.4it/s]
Backpropagation epoch: 0000000099 - current precision is 0.8179; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000100 - current precision is 0.7958; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000101 - current precision is 0.8057; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000102 - current precision is 0.8298; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000103 - current precision is 0.8361; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000104 - current precision is 0.8306; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000105 - current precision is 0.8557; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000106 - current precision is 0.8768; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000107 - current precision is 0.8565; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.5it/s]
Backpropagation epoch: 0000000108 - current precision is 0.8626; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000109 - current precision is 0.8673; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000110 - current precision is 0.8635; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000111 - current precision is 0.8598; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000112 - current precision is 0.8738; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  21.0it/s]
Backpropagation epoch: 0000000113 - current precision is 0.8722; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000114 - current precision is 0.8484; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000115 - current precision is 0.8821; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000116 - current precision is 0.8764; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.6it/s]
Backpropagation epoch: 0000000117 - current precision is 0.8717; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.8it/s]
Backpropagation epoch: 0000000118 - current precision is 0.8152; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.9it/s]
Backpropagation epoch: 0000000119 - current precision is 0.8867; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000120 - current precision is 0.8931; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00,  20.7it/s]
Backpropagation epoch: 0000000121 - current precision is 0.9089; aiming for precision >= 0.9000 in training: 100%|██████████| 1.00/1.00 [00:00<00:00, 32.3kit/s]

Model saved with 55273681 parameters. 
```
Now just load the model you want to adjust.
Note: In some cases, it may be necessary to reduce the number of repetitions in the adjustment loop so that the model does not forget the information from the base training. Increasing the number of repetitions in the loop too much may cause the model to consider the adjustment data more than the training data, or in some rarer cases, the model may only consider the adjustment data.
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
# load the model
gptmodel.loadModel('./models/my_model.gpt')
# applies fine-tuning to the model
for _ in range(10): # repeats the fine-tuning 10 times, if the inference is not satisfactory, repeat the fine-tuning increasing the number of loop repetitions, until you find a number of repetitions that provides satisfactory inferences
	gptmodel.addFit('What does the acronym GPT mean in language models?', 'GPT in language models stands for Generative Pre-trained Transformer.')
	gptmodel.addFit('What is the capital of Spain?', 'The capital of Spain is Madrid.')
# saves a new model with the fine-tuning applied
if gptmodel.saveModel('./models/adjusted_model.gpt'): print('Adjusted model saved successfully.')
else: print('Error applying fine tuning!!')

```
```bash
Adjusted model saved successfully.
```
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/adjusted_model.gpt') # loading of the adjusted model

gptmodel.print_predict(prompt='Hello! Who are you?', stream=True)
gptmodel.print_predict(prompt='What does the acronym GPT mean in language models?', stream=True)
gptmodel.print_predict(prompt='What is the capital of Spain?', stream=True)

```
```bash
Hello! I am Sapiens Chat, an AI model created by Sapiens Technology.                                                                                            
GPT in language models stands for Generative Pre-trained Transformer.
The capital of Spain is Madrid.
```
To obtain the result of the training metrics, simply capture the return of the "train" function.
Note: to facilitate understanding and standardization of metrics, all results will be forced to fit into a scale between 0 and 1, that is, all metrics will be obtained in percentage values ​​between 0 and 1. If any of them exceeds 1 in its real value, the assigned value will be computed with the limit of 1 and if the value is less than 0, the assigned value will be computed as 0.
```python
from gptmodel import GPTModel
# we are setting the following values in the below class: (embedding_dim: 256; block_size: 64; batch_size: 8; number_heads: 4; number_layers: 4; dropout: 0.2; learning_rate: 0.0001; eval_interval: 200; epochs: 500)
gptmodel = GPTModel(256, 64, 8, 4, 4, 0.2, 0.0001, 200, 500)
training_metrics = gptmodel.train('./dataset.json', precision=0.99, validate=0.01) # obtains a dictionary in the following format: {'val_loss': 0.0, 'loss': 0.0, 'generalization_rate': 0.0, 'precision': 0.0}
val_loss = training_metrics['val_loss'] # value of information loss during validation of test data (test data is separate and not part of training)
loss = training_metrics['loss'] # final loss of information, during the computation of the raw training data
generalization_rate = training_metrics['generalization_rate'] # rate referring to how well the model is able to generalize, that is, how much the model can adapt to inputs different from the inputs used in training
precision = training_metrics['precision'] # final reliability of the model that corresponds to the confidence rate in the responses for the future inferences (reliability of inferences)
# displays training metrics (display with eight decimal places)
print(f'Loss in validation test: {val_loss:.8f}')
print(f'Loss in training: {loss:.8f}')
print(f'Generalization ability: {generalization_rate:.8f}')
print(f'Model reliability: {precision:.8f}')
gptmodel.saveModel('./models/my_model.gpt')

```
```bash
Backpropagation epoch: 0000000001 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000002 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.4it/s]
Backpropagation epoch: 0000000003 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.5it/s]
Backpropagation epoch: 0000000004 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.7it/s]
Backpropagation epoch: 0000000005 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.7it/s]
Backpropagation epoch: 0000000006 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.5it/s]
Backpropagation epoch: 0000000007 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.7it/s]
Backpropagation epoch: 0000000008 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.6it/s]
Backpropagation epoch: 0000000009 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000010 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  28.4it/s]
Backpropagation epoch: 0000000011 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  27.6it/s]
Backpropagation epoch: 0000000012 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000013 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000014 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.0it/s]
Backpropagation epoch: 0000000015 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000016 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000017 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  28.6it/s]
Backpropagation epoch: 0000000018 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  27.9it/s]
Backpropagation epoch: 0000000019 - current precision is 0.4861; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  27.8it/s]
Backpropagation epoch: 0000000020 - current precision is 0.4788; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000021 - current precision is 0.9534; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  27.5it/s]
Backpropagation epoch: 0000000022 - current precision is 0.5744; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000023 - current precision is 0.0070; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000024 - current precision is 0.3772; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000025 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000026 - current precision is 0.2383; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000027 - current precision is 0.5798; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000028 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000029 - current precision is 0.7899; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000030 - current precision is 0.1921; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000031 - current precision is 0.6416; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000032 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000033 - current precision is 0.4761; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000034 - current precision is 0.4883; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000035 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000036 - current precision is 0.9793; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000037 - current precision is 0.2296; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000038 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000039 - current precision is 0.2732; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000040 - current precision is 0.9674; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000041 - current precision is 0.3545; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000042 - current precision is 0.9828; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000043 - current precision is 0.1600; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000044 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000045 - current precision is 0.4624; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000046 - current precision is 0.4326; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000047 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000048 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000049 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000050 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000051 - current precision is 0.1078; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000052 - current precision is 0.5258; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000053 - current precision is 0.0492; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.4it/s]
Backpropagation epoch: 0000000054 - current precision is 0.4948; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000055 - current precision is 0.4857; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.5it/s]
Backpropagation epoch: 0000000056 - current precision is 0.1073; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000057 - current precision is 0.5242; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000058 - current precision is 0.5639; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000059 - current precision is 0.1873; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000060 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000061 - current precision is 0.3135; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.3it/s]
Backpropagation epoch: 0000000062 - current precision is 0.3637; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000063 - current precision is 0.2498; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000064 - current precision is 0.4718; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000065 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000066 - current precision is 0.1569; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000067 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000068 - current precision is 0.7559; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.7it/s]
Backpropagation epoch: 0000000069 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000070 - current precision is 0.1169; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.8it/s]
Backpropagation epoch: 0000000071 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000072 - current precision is 0.6060; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.4it/s]
Backpropagation epoch: 0000000073 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.4it/s]
Backpropagation epoch: 0000000074 - current precision is 0.7421; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000075 - current precision is 0.7627; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000076 - current precision is 0.7797; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000077 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000078 - current precision is 0.3461; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000079 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.3it/s]
Backpropagation epoch: 0000000080 - current precision is 0.6191; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000081 - current precision is 0.5224; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.9it/s]
Backpropagation epoch: 0000000082 - current precision is 0.5652; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000083 - current precision is 0.2652; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000084 - current precision is 0.4901; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  29.6it/s]
Backpropagation epoch: 0000000085 - current precision is 0.3075; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000086 - current precision is 0.7097; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000087 - current precision is 0.6484; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000088 - current precision is 0.4509; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.1it/s]
Backpropagation epoch: 0000000089 - current precision is 0.1596; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000090 - current precision is 0.5589; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.0it/s]
Backpropagation epoch: 0000000091 - current precision is 0.0000; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  30.2it/s]
Backpropagation epoch: 0000000092 - current precision is 0.9943; aiming for precision >= 0.9900 in training: 100%|██████████| 21.0/21.0 [00:00<00:00,  753kit/s]

Loss in validation test: 0.47629535
Loss in training: 0.00565166
Generalization ability: 0.27460337
Model reliability: 0.99434834
```
```python
from gptmodel import GPTModel
gptmodel = GPTModel()
gptmodel.loadModel('./models/my_model.gpt')
gptmodel.print_predict(prompt='Hello!', stream=True)

```
```bash
 I am Sapiens Chat.
```


## Methods
### Construtor: GPTModel
Parameters (If not assigned, they will be configured automatically)
| Name                | Description                                                                                                                                                                                              | Type  | Default Value     |
|---------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|-------|-------------------|
| embedding_dim       | amount of elements per token vector, each token will be converted into a vector with that amount of numbers                                                                                              | int   | 384               |
| block_size          | maximum number of tokens for the context window size                                                                                                                                                     | int   | 500               |
| batch_size          | maximum number of input-output pairs read at once during training                                                                                                                                        | int   | 32                |
| number_heads        | number of attention mechanisms paying attention to different parts of the text at the same time                                                                                                          | int   | 6                 |
| number_layers       | number of hidden layers in the network, the higher this number, the greater the pattern abstraction                                                                                                      | int   | 6                 |
| dropout             | percentage (between 0 and 1) of neurons turned off at each step to prevent the model from memorizing the answers and actually learn instead of just memorizing                                           | float | 0.1               |
| learning_rate       | learning speed, if too high the model learns little and quickly, if too low it learns a lot but slowly (preferably between 0 and 1)                                                                      | float | 3e-4              |
| eval_interval       | interval between one step and another used to update the validation loss rate                                                                                                                            | int   | 500               |
| epochs              | number of times all data is read for learning, if too high the model runs the risk of starting to memorize instead of learning                                                                           | int   | 2000              |

### train (function return type: dict): Returns a dictionary with the training metrics.
Parameters
| Name                | Description                                                                                                                                                                                              | Type  | Default Value     |
|---------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|-------|-------------------|
| dataset_path        | path to a local or web dataset in txt or preferably json format                                                                                                                                          | str   | ''                |
| string              | text to train the model, or to complement the dataset                                                                                                                                                    | str   | ''                |
| precision           | percentage accuracy or reliability between 0 and 1 desired during weights adjustment, which when achieved will end the training                                                                          | float | 0.9               |
| tokenizer           | type of encoder used in converting text into sequence of embeddings (gpt to encode chunks of words, or sapi to encode each character individually)                                                       | str   | 'gpt'             |
| context_window      | maximum number of tokens accepted in the inference prompt                                                                                                                                                | int   | None              |
| end_tag             | defines what subtext in the training data will be used to separate input and output pairs from each other                                                                                                | str   | None              |
| validate            | percentage between 0 and 1 relative to the amount of training data separated for validation and testing                                                                                                  | float | 0.0               |
| progress            | if True enables visualization of training progress, if False disables visualization of training progress                                                                                                 | bool  | True              |

### saveModel (function return type: bool): Returns True if the model is saved successfully, or False otherwise.
Parameters
| Name                | Description                                                                                                                                                                                              | Type  | Default Value     |
|---------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|-------|-------------------|
| model_path          | local path with directory and name for the trained model generation                                                                                                                                      | str   | ''                |
| progress            | if True enables the progress bar that displays the current stage of saving, if False will hide the progress bar                                                                                          | bool  | True              |

### loadModel (function return type: bool): Returns True if the model is loaded successfully, or False otherwise.
Parameters
| Name                | Description                                                                                                                                                                                              | Type  | Default Value     |
|---------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|-------|-------------------|
| model_path          | local path with the directory and file name of the model that will be loaded                                                                                                                             | str   | ''                |
| progress            | if True enables the progress bar that displays the current stage of loading, if False will hide the progress bar                                                                                         | bool  | True              |

### addFit (function return type: bool): Returns True if an of "input/prompt/instruction/question" and "output/answer" pair is successfully added to the current model, or False otherwise.
Parameters
| Name                | Description                                                                                                                                                                                              | Type  | Default Value     |
|---------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|-------|-------------------|
| prompt              | input for which you want to obtain an output                                                                                                                                                             | str   | ''                |
| answer              | desired output for when a similar input is made to the inference                                                                                                                                         | str   | ''                |

### predict (function return type: str/generator object): Returns a string with the response to the user prompt.
Parameters
| Name                | Description                                                                                                                                                                                              | Type  | Default Value     |
|---------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|-------|-------------------|
| prompt              | input for which you want to infer a response                                                                                                                                                             | str   | ''                |
| max_tokens          | approximate maximum number of tokens in the response text                                                                                                                                                | int   | 500               |
| temperature         | percentage (between 0 and 1) for how different one answer will be from another when the same input/question is asked (the answers will remain the same when the training dataset is small)               | float | 0.5               |
| top_k               | amount of most likely tokens considered for the random selection of response tokens                                                                                                                      | int   | 0                 |
| top_p               | minimum probability used to select the most likely tokens in the response                                                                                                                                | float | 1.0               |
| stream              | if True, returns a token generator to obtain one token at a time with the response being generated in real time, if False, will return the complete response  only when all tokens are generated         | bool  | False             |

### print_predict (method without return): Displays a string with the model prediction/inference result.
Parameters
| Name                | Description                                                                                                                                                                                              | Type  | Default Value     |
|---------------------|----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|-------|-------------------|
| prompt              | input for which you want to display a response                                                                                                                                                           | str   | ''                |
| max_tokens          | approximate maximum number of tokens in the response text                                                                                                                                                | int   | 500               |
| temperature         | percentage (between 0 and 1) for how different one answer will be from another when the same input/question is asked (the answers will remain the same when the training dataset is small)               | float | 0.5               |
| top_k               | amount of most likely tokens considered for the random selection of response tokens                                                                                                                      | int   | 0                 |
| top_p               | minimum probability used to select the most likely tokens in the response                                                                                                                                | float | 1.0               |
| stream              | if True, will display the response in real time with one token at a time, if False, will display the complete response only when all tokens are generated                                                | bool  | False             |


## Contributing

We do not accept contributions that may result in changing the original code.

Make sure you are using the appropriate version.

## License

This is proprietary software and its alteration and/or distribution without the developer's authorization is not permitted.
